/* BDF - Copyright (c) 2009-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstruction;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.interaction.Domain;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.session.LoginException;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.get.GetInstructionFactory;
import fr.exemole.bdfserver.multi.MultiConf;
import fr.exemole.bdfserver.multi.MultiConstants;
import fr.exemole.bdfserver.multi.MultiResponseHandlerFactory;
import fr.exemole.bdfserver.servlets.instructions.AppInstruction;
import fr.exemole.bdfserver.servlets.instructions.DomainInstruction;
import fr.exemole.bdfserver.servlets.instructions.RunInstruction;
import fr.exemole.bdfserver.servlets.instructions.SessionInstruction;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.localisation.LangPreference;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.OutputInfo;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.servlets.HttpAccessException;
import net.mapeadores.util.servlets.HttpServletRequestMap;
import net.mapeadores.util.servlets.MultiPartParser;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.CommandMessageResponseHandler;
import net.mapeadores.util.servlets.multipart.DefaultMultiPartParser;


/**
 *
 * @author Vincent Calame
 */
public class BDFServlet extends HttpServlet {

    private final MultiPartParser multiPartParser = new DefaultMultiPartParser();

    @Override
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        try {
            processRequest(request, response);
        } catch (HttpAccessException hae) {
            sendError(request, response, hae, null, null, null);
        }
    }

    @Override
    public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        try {
            processRequest(request, response);
        } catch (HttpAccessException hae) {
            sendError(request, response, hae, null, null, null);
        }
    }

    private void processRequest(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        String pathInfo = request.getPathInfo();
        if (pathInfo == null) {
            throw new HttpAccessException(HttpAccessException.NOT_FOUND);
        }
        BDFWebapps bdfWebapps = (BDFWebapps) getServletContext().getAttribute(BDFWebapps.BDFWEBAPPS_KEY);
        if (pathInfo.equals("/")) {
            if (bdfWebapps.isMultiBdf()) {
                MultiConf multiConf = bdfWebapps.getMultiConf();
                multiConf.update();
                String defaultFichothequeName = multiConf.getDefaultFichothequeName();
                if (defaultFichothequeName != null) {
                    sendRedirect(response, defaultFichothequeName + "/" + Domains.SESSION);
                } else {
                    sendRedirect(response, "_admin");
                }
            } else {
                sendRedirect(response, Domains.SESSION);
            }
            return;
        }
        pathInfo = pathInfo.substring(1);
        BdfServer bdfServer;
        if (bdfWebapps.isMultiBdf()) {
            if (pathInfo.startsWith("_")) {
                if (pathInfo.equals("_admin")) {
                    processMultiAdminRequest(bdfWebapps, request, response);
                    return;
                } else if (pathInfo.equals("_admin_noscript")) {
                    processMultiAdminNoscriptRequest(bdfWebapps, request, response);
                    return;
                } else if (pathInfo.startsWith("_rsc/")) {
                    String path = pathInfo.substring("_rsc/".length());
                    ResponseHandler responseHandler = MultiResponseHandlerFactory.getResourceResponseHandler(bdfWebapps, path);
                    if (responseHandler != null) {
                        responseHandler.handleResponse(response);
                        return;
                    }
                }
                throw new HttpAccessException(HttpAccessException.NOT_FOUND);
            } else if (pathInfo.equals(Domains.SESSION)) {
                MultiConf multiConf = bdfWebapps.getMultiConf();
                multiConf.update();
                String defaultFichothequeName = multiConf.getDefaultFichothequeName();
                if (defaultFichothequeName != null) {
                    sendRedirect(response, defaultFichothequeName + "/" + Domains.SESSION);
                    return;
                } else {
                    throw new HttpAccessException(HttpAccessException.NOT_FOUND);
                }
            } else {
                int idx = pathInfo.indexOf('/');
                if (idx == -1) {
                    sendRedirect(response, pathInfo + "/" + Domains.SESSION);
                    return;
                } else {
                    bdfServer = bdfWebapps.getBdfServer(pathInfo.substring(0, idx));
                    if (bdfServer == null) {
                        throw new HttpAccessException(HttpAccessException.NOT_FOUND);
                    }
                    if (idx == (pathInfo.length() - 1)) {
                        sendRedirect(response, Domains.SESSION);
                        return;
                    }
                    pathInfo = pathInfo.substring(idx + 1);
                }
            }
        } else {
            bdfServer = bdfWebapps.getBdfServer(null);
        }
        processRequest(bdfWebapps, bdfServer, pathInfo, request, response);
    }

    private void processMultiAdminRequest(BDFWebapps bdfWebapps, HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        RequestMap requestMap = new HttpServletRequestMap(request, multiPartParser);
        OutputInfo outputInfo = OutputInfo.buildFromRequest(requestMap, "adminjs");
        MultiConf multiConf = bdfWebapps.getMultiConf();
        if (outputInfo.getType() == OutputInfo.PAGE_TYPE) {
            multiConf.update();
            if (requestMap.isTrue(MultiConstants.RELOAD_PARAMNAME)) {
                bdfWebapps.getWebappsJsAnalyser().clearCache();
            }
        }
        ResponseHandler responseHandler;
        CommandMessage initErrorMessage = multiConf.getInitErrorMessage();
        if (initErrorMessage != null) {
            if (outputInfo.getType() == OutputInfo.JSON_TYPE) {
                responseHandler = CommandMessageResponseHandler.getJsonInstance(multiConf.getMessageLocalisation(), requestMap, initErrorMessage);
            } else if (initErrorMessage.getMessageKey().equals(MultiConf.MISSING_FILE_ERROR)) {
                responseHandler = MultiResponseHandlerFactory.getIniResponseHandler(bdfWebapps, requestMap);
            } else {
                responseHandler = CommandMessageResponseHandler.getHtmlInstance(multiConf.getMessageLocalisation(), initErrorMessage);
            }
        } else {
            switch (outputInfo.getType()) {
                case OutputInfo.JSON_TYPE:
                    short check = bdfWebapps.checkMultiAdmin(request);
                    responseHandler = MultiResponseHandlerFactory.getJsonRespondeHandler(bdfWebapps, check, requestMap, outputInfo.getOutput());
                    break;
                case OutputInfo.STREAM_TYPE:
                    responseHandler = MultiResponseHandlerFactory.getStreamRespondeHandler(bdfWebapps, outputInfo.getOutput());
                    break;
                default:
                    responseHandler = MultiResponseHandlerFactory.getAdminJsRespondeHandler(bdfWebapps);
            }
        }
        responseHandler.handleResponse(response);
    }

    private void processMultiAdminNoscriptRequest(BDFWebapps bdfWebapps, HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        MultiConf multiConf = bdfWebapps.getMultiConf();
        multiConf.update();
        CommandMessage initErrorMessage = multiConf.getInitErrorMessage();
        ResponseHandler responseHandler = null;
        if (initErrorMessage != null) {
            responseHandler = CommandMessageResponseHandler.getHtmlInstance(multiConf.getMessageLocalisation(), initErrorMessage);
        } else {
            short check = bdfWebapps.checkMultiAdmin(request);
            RequestMap requestMap = new HttpServletRequestMap(request, multiPartParser);
            responseHandler = MultiResponseHandlerFactory.getAdminNoscriptRespondeHandler(bdfWebapps, check, requestMap);
        }
        responseHandler.handleResponse(response);
    }

    private void processRequest(BDFWebapps bdfWebapps, BdfServer bdfServer, String pathInfo, HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        RequestMap requestMap = new HttpServletRequestMap(request, multiPartParser);
        if (bdfServer.getInitState() == BdfServer.NOADMIN_STATE) {
            HttpAccessException exception = new HttpAccessException(HttpAccessException.INTERNAL_ERROR, "No Admin State", LogUtils.error("_ error.unsupported.noadminstate"));
            sendError(request, response, exception, requestMap, bdfServer, null);
            return;
        }
        BdfInstruction bdfInstruction = getBdfInstruction(bdfServer, requestMap, pathInfo);
        if (bdfInstruction == null) {
            HttpAccessException exception = new HttpAccessException(HttpAccessException.NOT_FOUND);
            sendError(request, response, exception, requestMap, bdfServer, null);
            return;
        }
        BdfUser bdfUser = null;
        short needBdfUser = bdfInstruction.getBdfUserNeed();
        if (needBdfUser != BdfInstructionConstants.NONE_BDFUSER) {
            try {
                bdfUser = bdfWebapps.getBdfUser(bdfServer, requestMap, request);
                if (needBdfUser == BdfInstructionConstants.EXIT_BDFUSER) {
                    bdfWebapps.invalidateBdfUser(bdfServer, bdfUser);
                    throw new LoginException(LoginException.EXIT_ERROR);
                }
            } catch (LoginException le) {
                if (needBdfUser != BdfInstructionConstants.IF_ANY_BDFUSER) {
                    ResponseHandler responseHandler = LoginEngine.resolveLoginException(bdfServer, requestMap, pathInfo, bdfInstruction, le);
                    responseHandler.handleResponse(response);
                    return;
                }
            }
        }
        try {
            ResponseHandler responseHandler = bdfInstruction.runInstruction(bdfUser);
            if (responseHandler == null) {
                throw new HttpAccessException(HttpAccessException.NOT_FOUND);
            }
            long lastModified = responseHandler.getLastModified();
            if (lastModified == ResponseHandler.LASTMODIFIED_NOCACHE) {
                response.setHeader("Cache-Control", "no-store");
            } else if (lastModified > 0) {
                response.setDateHeader("Last-Modified", lastModified);
                try {
                    long since = request.getDateHeader("If-Modified-Since");
                    if (since >= lastModified) {
                        response.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
                        return;
                    }
                } catch (IllegalArgumentException iae) {
                }
            }

            responseHandler.handleResponse(response);
        } catch (HttpAccessException hae) {
            sendError(request, response, hae, requestMap, bdfServer, bdfUser);
        }
    }

    private void sendError(HttpServletRequest request, HttpServletResponse response, HttpAccessException hae, RequestMap requestMap, BdfServer bdfServer, BdfUser bdfUser) throws IOException {
        CommandMessage commandMessage = hae.getCommandMessage();
        String message = null;
        if ((commandMessage != null) && (bdfServer != null)) {
            LangPreference langPreference;
            if (bdfUser == null) {
                langPreference = BdfInstructionUtils.getLangPreference(requestMap, bdfServer);
            } else {
                langPreference = bdfUser.getLangPreference();
            }
            message = bdfServer.getL10nManager().getMessageLocalisation(langPreference).toString(commandMessage);
        }
        if (message == null) {
            message = hae.getMessage();
        }
        if (message == null) {
            message = request.getServletPath() + request.getPathInfo();
        }
        response.sendError(convert(hae), message);
    }

    private static int convert(HttpAccessException hae) {
        switch (hae.getHttpType()) {
            case HttpAccessException.BAD_REQUEST:
                return HttpServletResponse.SC_BAD_REQUEST;
            case HttpAccessException.NOT_FOUND:
                return HttpServletResponse.SC_NOT_FOUND;
            case HttpAccessException.FORBIDDEN_ACCESS:
                return HttpServletResponse.SC_FORBIDDEN;
            case HttpAccessException.INTERNAL_ERROR:
                return HttpServletResponse.SC_INTERNAL_SERVER_ERROR;
            default:
                return HttpServletResponse.SC_INTERNAL_SERVER_ERROR;
        }
    }

    private void sendRedirect(HttpServletResponse response, String path) throws IOException {
        response.sendRedirect(path);
    }

    private static BdfInstruction getBdfInstruction(BdfServer bdfServer, RequestMap requestMap, String pathInfo) {
        int idx = pathInfo.indexOf('/');
        if (idx == -1) {
            Domain domain = parsePathInfo(pathInfo);
            return getDomainBdfInstruction(bdfServer, requestMap, domain);
        } else {
            String first = pathInfo.substring(0, idx);
            String next = pathInfo.substring(idx + 1);
            return GetInstructionFactory.getBdfInstruction(bdfServer, requestMap, first, next);
        }
    }

    private static BdfInstruction getDomainBdfInstruction(BdfServer bdfServer, RequestMap requestMap, Domain domain) {
        switch (domain.getFirstPart()) {
            case Domains.SESSION:
                return new SessionInstruction(bdfServer, requestMap);
            case Domains.RUN:
                return new RunInstruction(bdfServer, requestMap);
            case Domains.APP:
                return AppInstruction.build(bdfServer, requestMap, domain.getSecondPart());
            case Domains.ADMINISTRATION:
            case Domains.ALBUM:
            case Domains.CONFIGURATION:
            case Domains.CORPUS:
            case Domains.MAILING:
            case Domains.EXPORTATION:
            case Domains.EDITION:
            case Domains.SELECTION:
            case Domains.SPHERE:
            case Domains.THESAURUS:
            case Domains.MAIN:
            case Domains.MISC:
            case Domains.ADDENDA:
            case Domains.IMPORTATION:
            case Domains.PIOCHE:
            case Domains.EXTENSION:
                return new DomainInstruction(bdfServer, requestMap, domain);
            default:
                return null;
        }
    }

    private static Domain parsePathInfo(String pathInfo) {
        String firstPart = pathInfo.toLowerCase();
        String secondPart = "";
        int length = pathInfo.length();
        for (int i = 0; i < length; i++) {
            char carac = pathInfo.charAt(i);
            boolean isSeparator = false;
            switch (carac) {
                case ':':
                case '_':
                case '-':
                    isSeparator = true;
                    break;
            }
            if (isSeparator) {
                secondPart = firstPart.substring(i + 1);
                firstPart = firstPart.substring(0, i);
                break;
            }
        }
        return new Domain(firstPart, secondPart);
    }

}
