/* BdfServer - Copyright (c) 2013-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.importation.engines;

import fr.exemole.bdfserver.api.EditSession;
import fr.exemole.bdfserver.tools.configuration.ConfigurationUtils;
import net.fichotheque.ExistingIdException;
import net.fichotheque.FichothequeEditor;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.CorpusEditor;
import net.fichotheque.corpus.FicheChange;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.NoMasterIdException;
import net.fichotheque.corpus.fiche.Fiche;
import net.fichotheque.croisement.CroisementEditor;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.format.FormatContext;
import net.fichotheque.importation.CorpusImport;
import net.fichotheque.tools.FichothequeTools;
import net.fichotheque.tools.corpus.CorpusTools;
import net.fichotheque.tools.corpus.FieldGenerationEngine;
import net.fichotheque.utils.CorpusUtils;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.FuzzyDate;


/**
 *
 * @author Vincent Calame
 */
public final class CorpusImportEngine {

    private CorpusImportEngine() {
    }

    public static void runCorpusImport(EditSession editSession, ExtractionContext extractionContext, CorpusImport corpusImport) {
        String type = corpusImport.getType();
        if (type.equals(CorpusImport.CREATION_TYPE)) {
            runCreation(editSession, extractionContext, corpusImport);
        } else if (type.equals(CorpusImport.CHANGE_TYPE)) {
            runChange(editSession, extractionContext, corpusImport);
        } else if (type.equals(CorpusImport.REMOVE_TYPE)) {
            runRemove(editSession, corpusImport);
        }
    }

    private static void runCreation(EditSession editSession, ExtractionContext extractionContext, CorpusImport corpusImport) {
        Corpus corpus = corpusImport.getCorpus();
        FichothequeEditor fichothequeEditor = editSession.getFichothequeEditor();
        CroisementEditor croisementEditor = fichothequeEditor.getCroisementEditor();
        CorpusEditor corpusEditor = fichothequeEditor.getCorpusEditor(corpus.getSubsetKey());
        FormatContext formatContext = editSession.getBdfServer().getFormatContext();
        Lang defaultLang = ConfigurationUtils.getDefaultWorkingLang(editSession.getBdfServer().getLangConfiguration());
        int ficheImportCount = corpusImport.getFicheImportCount();
        FieldGenerationEngine engine = FieldGenerationEngine.build(corpus, extractionContext, formatContext, defaultLang);
        for (int i = 0; i < ficheImportCount; i++) {
            CorpusImport.CreationFicheImport ficheImport = (CorpusImport.CreationFicheImport) corpusImport.getFicheImport(i);
            FicheChange ficheChange = ficheImport.getFicheChange();
            FicheMeta ficheMeta;
            try {
                ficheMeta = corpusEditor.createFiche(ficheImport.getNewId());
                FuzzyDate creationDate = ficheImport.getCreationDate();
                if (creationDate != null) {
                    corpusEditor.setDate(ficheMeta, creationDate, false);
                    corpusEditor.setDate(ficheMeta, FuzzyDate.current(), true);
                } else {
                    corpusEditor.setDate(ficheMeta, FuzzyDate.current(), false);
                }
            } catch (ExistingIdException | NoMasterIdException e) {
                continue;
            }
            FichothequeUtils.changeAttributes(fichothequeEditor, ficheMeta, ficheImport.getAttributeChange());
            LiensImportEngine.run(croisementEditor, ficheMeta, ficheImport.getLiensImport(), extractionContext);
            Fiche fiche = new Fiche();
            if (CorpusUtils.hasChanges(ficheChange)) {
                CorpusUtils.updateFiche(fiche, ficheChange);
            }
            CorpusTools.saveFiche(corpusEditor, ficheMeta, fiche, engine, false);
        }
    }

    private static void runChange(EditSession editSession, ExtractionContext extractionContext, CorpusImport corpusImport) {
        Corpus corpus = corpusImport.getCorpus();
        FichothequeEditor fichothequeEditor = editSession.getFichothequeEditor();
        CroisementEditor croisementEditor = fichothequeEditor.getCroisementEditor();
        CorpusEditor corpusEditor = fichothequeEditor.getCorpusEditor(corpus.getSubsetKey());
        int ficheImportCount = corpusImport.getFicheImportCount();
        FormatContext formatContext = editSession.getBdfServer().getFormatContext();
        Lang defaultLang = ConfigurationUtils.getDefaultWorkingLang(editSession.getBdfServer().getLangConfiguration());
        FieldGenerationEngine engine = FieldGenerationEngine.build(corpus, extractionContext, formatContext, defaultLang);
        for (int i = 0; i < ficheImportCount; i++) {
            CorpusImport.ChangeFicheImport ficheImport = (CorpusImport.ChangeFicheImport) corpusImport.getFicheImport(i);
            FicheMeta ficheMeta = ficheImport.getFicheMeta();
            FichothequeUtils.changeAttributes(fichothequeEditor, ficheMeta, ficheImport.getAttributeChange());
            LiensImportEngine.run(croisementEditor, ficheMeta, ficheImport.getLiensImport(), extractionContext);
            FicheChange ficheChange = ficheImport.getFicheChange();
            FuzzyDate creationDate = ficheImport.getCreationDate();
            if (creationDate != null) {
                corpusEditor.setDate(ficheMeta, creationDate, false);
            }
            if (CorpusUtils.hasChanges(ficheChange)) {
                Fiche fiche = corpus.getFiche(ficheMeta);
                CorpusUtils.updateFiche(fiche, ficheChange);
                CorpusTools.saveFiche(corpusEditor, ficheMeta, fiche, engine, true);
            }
        }
    }

    private static void runRemove(EditSession editSession, CorpusImport corpusImport) {
        Corpus corpus = corpusImport.getCorpus();
        FichothequeEditor fichothequeEditor = editSession.getFichothequeEditor();
        CorpusEditor corpusEditor = fichothequeEditor.getCorpusEditor(corpus.getSubsetKey());
        int ficheImportCount = corpusImport.getFicheImportCount();
        for (int i = 0; i < ficheImportCount; i++) {
            CorpusImport.FicheImport ficheImport = (CorpusImport.FicheImport) corpusImport.getFicheImport(i);
            FicheMeta ficheMeta = ficheImport.getFicheMeta();
            FichothequeTools.prepareToRemove(fichothequeEditor, ficheMeta);
            corpusEditor.removeFiche(ficheMeta);
        }
    }

}
