/* BdfServer_Commands - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.util.ArrayList;
import java.util.List;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.CorpusMetadata;
import net.fichotheque.corpus.metadata.CorpusMetadataEditor;
import net.fichotheque.corpus.metadata.FieldGeneration;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.corpus.metadata.FieldOptionException;
import net.fichotheque.corpus.metadata.FieldOptionsConstants;
import net.fichotheque.tools.corpus.FieldGenerationParser;
import net.fichotheque.utils.FieldOptionsUtils;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangsUtils;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.logging.MessageByLine;
import net.mapeadores.util.logging.SimpleLineMessageHandler;
import net.mapeadores.util.money.CurrenciesUtils;
import net.mapeadores.util.money.ExtendedCurrency;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class FieldOptionsCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "FieldOptions";
    public final static String COMMANDKEY = "_ CRP-07";
    public final static String DEFAULTSPHERE_SUFFIX = "|defaultsphere";
    public final static String BLOCKDISPLAY_SUFFIX = "|blockdisplay";
    public final static String SUBFIELDDISPLAY_SUFFIX = "|subfielddisplay";
    public final static String CURRENCIES_SUFFIX = "|currencies";
    public final static String BASEURL_SUFFIX = "|baseurl";
    public final static String FIELDGENERATION_PARAMNAME = "fieldgeneration";
    public final static String ADDRESSFIELDARRAY_SUFFIX = "|addressfieldarray";
    public final static String GEOLOCALISATIONFIELD_PARAMNAME = "geolocalisationfield";
    public final static String LANGSCOPE_SUFFIX = "|langscope";
    public final static String LANGS_SUFFIX = "|langs";
    public final static String ALL_LANGSCOPE = "all";
    public final static String CONFIG_LANGSCOPE = "config";
    public final static String MULTILANG_LANGSCOPE = "mul";
    public final static String NOLINGUISTICCONTENT_LANGSCOPE = "zxx";
    public final static String LIST_LANGSCOPE = "list";
    private final static Lang[] MULTILANG_ARRAY = {Lang.build("mul")};
    private final static Lang[] NOLINGUISTICCONTENT_ARRAY = {Lang.build("zxx")};
    private Corpus corpus;
    private boolean initGeolocalisationField = false;
    private CorpusField geolocalisationField = null;
    private FieldGeneration fieldGeneration;


    public FieldOptionsCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        initEditSession(Domains.CORPUS + "/" + COMMANDNAME);
        putResultObject(CORPUS_OBJ, corpus);
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        CorpusMetadataEditor corpusMetadataEditor = editSession.getFichothequeEditor().getCorpusEditor(corpus.getSubsetKey()).getCorpusMetadataEditor();
        boolean done = false;
        if (fieldGeneration != null) {
            boolean fieldGenerationDone = corpusMetadataEditor.setFieldGeneration(fieldGeneration);
            if (fieldGenerationDone) {
                done = true;
            }
        }
        if (initGeolocalisationField) {
            boolean geoDone;
            try {
                geoDone = corpusMetadataEditor.setGeolocalisationField(geolocalisationField);
                if (geoDone) {
                    done = true;
                }
            } catch (IllegalArgumentException iae) {
                throw new ImplementationException(iae);
            }
        }
        for (CorpusField corpusField : corpusMetadata.getProprieteList()) {
            if (setOptions(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        for (CorpusField corpusField : corpusMetadata.getInformationList()) {
            if (setOptions(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        CorpusField langCorpusField = corpusMetadata.getCorpusField(FieldKey.LANG);
        boolean langDone = setLangScope(corpusMetadataEditor, langCorpusField);
        if (langDone) {
            done = true;
        }
        if (done) {
            setDone("_ done.corpus.fieldoptions");
            bdfServer.getTransformationManager().clearDefaultTransformer();
        }
    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        corpus = BdfInstructionUtils.getMandatoryCorpus(fichotheque, requestMap);
        checkSubsetAdminPermission(corpus);
        CorpusMetadata corpusMetadata = corpus.getCorpusMetadata();
        String fieldGenerationRawString = requestMap.getParameter(FIELDGENERATION_PARAMNAME);
        if (fieldGenerationRawString != null) {
            SimpleLineMessageHandler lineMessageHandler = new SimpleLineMessageHandler();
            fieldGeneration = FieldGenerationParser.parse(fieldGenerationRawString, lineMessageHandler);
            if (lineMessageHandler.hasMessage()) {
                for (MessageByLine messageByLine : lineMessageHandler.toMessageByLineList()) {
                    for (MessageByLine.Category category : messageByLine.getCategoryList()) {
                        for (Message message : category.getMessageList()) {
                            addMultiError(message);
                        }
                    }
                }
                setMultiErrorTitle("_ error.list.options");
                return false;
            }
        }
        String geolocalisationFieldString = requestMap.getParameter(GEOLOCALISATIONFIELD_PARAMNAME);
        if (geolocalisationFieldString == null) {
            initGeolocalisationField = false;
        } else {
            initGeolocalisationField = true;
            if (geolocalisationFieldString.length() == 0) {
                geolocalisationField = null;
            } else {
                boolean done = false;
                try {
                    FieldKey fieldKey = FieldKey.parse(geolocalisationFieldString);
                    if (fieldKey.isProprieteFieldKey()) {
                        CorpusField corpusField = corpusMetadata.getCorpusField(fieldKey);
                        if ((corpusField != null) && (corpusField.getFicheItemType() == CorpusField.GEOPOINT_FIELD)) {
                            done = true;
                            geolocalisationField = corpusField;
                        }
                    }
                } catch (java.text.ParseException pe) {
                }
                if (!done) {
                    throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(GEOLOCALISATIONFIELD_PARAMNAME, geolocalisationFieldString));
                }
            }
        }
        return true;
    }

    private boolean setOptions(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField) {
        boolean done = false;
        if (FieldOptionsUtils.testFieldOption(FieldOptionsConstants.DEFAULTSPHEREKEY_OPTION, corpusField)) {
            if (setDefaultSphere(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        if (FieldOptionsUtils.testFieldOption(FieldOptionsConstants.INFORMATIONDISPLAY_OPTION, corpusField)) {
            if (setDisplay(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        if (FieldOptionsUtils.testFieldOption(FieldOptionsConstants.SUBFIELDDISPLAY_OPTION, corpusField)) {
            if (setSubfieldDisplay(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        if (FieldOptionsUtils.testFieldOption(FieldOptionsConstants.CURRENCYARRAY_OPTION, corpusField)) {
            if (setCurrencies(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        if (FieldOptionsUtils.testFieldOption(FieldOptionsConstants.ADDRESSFIELDARRAY_OPTION, corpusField)) {
            if (setAddressFieldArray(corpusMetadataEditor, corpusField)) {
                done = true;
            }
        }
        if (FieldOptionsUtils.testFieldOption(FieldOptionsConstants.BASEURL_OPTION, corpusField)) {
            if (setStringOption(corpusMetadataEditor, corpusField, FieldOptionsConstants.BASEURL_OPTION, BASEURL_SUFFIX)) {
                done = true;
            }
        }
        return done;
    }

    private boolean setDisplay(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField) {
        short value = FieldOptionsConstants.INLINE_DISPLAY;
        if (requestMap.isTrue(corpusField.getFieldKey().toString() + BLOCKDISPLAY_SUFFIX)) {
            value = FieldOptionsConstants.BLOCK_DISPLAY;
        }
        boolean done = false;
        try {
            done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.INFORMATIONDISPLAY_OPTION, value);
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setDefaultSphere(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField) {
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + DEFAULTSPHERE_SUFFIX);
        if (param == null) {
            return false;
        }
        boolean done = false;
        SubsetKey defaultSphereKey = null;
        if (param.length() > 0) {
            try {
                defaultSphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, param);
            } catch (java.text.ParseException pe) {
            }
        }
        try {
            done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.DEFAULTSPHEREKEY_OPTION, defaultSphereKey);
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setSubfieldDisplay(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField) {
        boolean done = false;
        try {
            if (requestMap.isTrue(corpusField.getFieldKey().toString() + SUBFIELDDISPLAY_SUFFIX)) {
                done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.SUBFIELDDISPLAY_OPTION, Boolean.TRUE);
            } else {
                done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.SUBFIELDDISPLAY_OPTION, null);
            }
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setStringOption(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField, String optionName, String suffix) {
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + suffix);
        try {
            boolean done = corpusMetadataEditor.setFieldOption(corpusField, optionName, param);
            return done;
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
    }

    private boolean setAddressFieldArray(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField) {
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + ADDRESSFIELDARRAY_SUFFIX);
        boolean done = false;
        try {
            if (param == null) {
                done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.ADDRESSFIELDARRAY_OPTION, null);
            } else {
                List<FieldKey> fieldKeyList = new ArrayList<FieldKey>();
                String[] tokenArray = StringUtils.getTechnicalTokens(param, false);
                int length = tokenArray.length;
                for (int i = 0; i < length; i++) {
                    try {
                        FieldKey fieldKey = FieldKey.parse(tokenArray[i]);
                        fieldKeyList.add(fieldKey);
                    } catch (java.text.ParseException pe) {
                    }
                }
                done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.ADDRESSFIELDARRAY_OPTION, fieldKeyList.toArray(new FieldKey[fieldKeyList.size()]));
            }
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setCurrencies(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField) {
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + CURRENCIES_SUFFIX);
        boolean done = false;
        try {
            ExtendedCurrency[] currencyArray = null;
            if (param != null) {
                currencyArray = CurrenciesUtils.toCleanCurrencyArray(param);
                if (currencyArray.length == 0) {
                    currencyArray = null;
                }
            }
            done = corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.CURRENCYARRAY_OPTION, currencyArray);
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;
    }

    private boolean setLangScope(CorpusMetadataEditor corpusMetadataEditor, CorpusField corpusField) {
        String param = requestMap.getParameter(corpusField.getFieldKey().toString() + LANGSCOPE_SUFFIX);
        if (param == null) {
            return false;
        }
        Short langScope;
        Lang[] langArray = null;
        switch (param) {
            case ALL_LANGSCOPE:
                langScope = FieldOptionsConstants.ALL_SCOPE;
                break;
            case CONFIG_LANGSCOPE:
                langScope = FieldOptionsConstants.CONFIG_SCOPE;
                break;
            case MULTILANG_LANGSCOPE:
                langScope = FieldOptionsConstants.LIST_SCOPE;
                langArray = MULTILANG_ARRAY;
                break;
            case NOLINGUISTICCONTENT_LANGSCOPE:
                langScope = FieldOptionsConstants.LIST_SCOPE;
                langArray = NOLINGUISTICCONTENT_ARRAY;
                break;
            case LIST_LANGSCOPE:
                langScope = FieldOptionsConstants.LIST_SCOPE;
                String langsString = requestMap.getParameter(corpusField.getFieldKey().toString() + LANGS_SUFFIX);
                if (langsString != null) {
                    langArray = LangsUtils.toCleanLangArray(langsString);
                    if (langArray.length == 0) {
                        langArray = null;
                    }
                }
                if (langArray == null) {
                    langScope = FieldOptionsConstants.CONFIG_SCOPE;
                }
                break;
            default:
                return false;

        }
        boolean done = false;
        try {
            boolean done1 = corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.LANGSCOPE_OPTION, langScope);
            boolean done2 = corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.LANGARRAY_OPTION, langArray);
            if ((done1) || (done2)) {
                done = true;
            }
        } catch (FieldOptionException foe) {
            throw new ImplementationException(foe);
        }
        return done;

    }


}
