/* BdfServer_Commands - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.exportation;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.storage.TemplateStorage;
import fr.exemole.bdfserver.tools.exportation.transformation.compilers.TemplateCompilerUtils;
import fr.exemole.bdfserver.tools.exportation.transformation.defaulttemplates.TemplateStorageUnitFactory;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.io.IOException;
import java.text.ParseException;
import net.fichotheque.EditOrigin;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.exportation.transformation.TemplateDescription;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.exportation.transformation.TransformationDescription;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.text.ValidExtension;


/**
 *
 * @author Vincent Calame
 */
public class TemplateCreationCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "TemplateCreation";
    public final static String COMMANDKEY = "_ EXP-31";
    public final static String NEW_TEMPLATE_PARAMNAME = "newtemplate";
    public final static String WITHIF_PARAMNAME = "withif";
    public final static String TYPE_PARAMNAME = "type";
    public final static String HTML_XSLT_TYPE_PARAMVALUE = "html_xslt";
    public final static String ODT_XSLT_TYPE_PARAMVALUE = "odt_xslt";
    public final static String ODT_PROPERTIES_TYPE_PARAMVALUE = "odt_properties";
    public final static String WITHEXTRACTION_PARAMNAME = "withextraction";
    public final static String TABLEXPORT_PARAMNAME = "tableexportname";
    private TemplateKey templateKey;
    private String creationType;
    private String templateType;
    private String tableExportName;
    private boolean withIf = false;
    private boolean withExtraction = false;

    public TemplateCreationCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        EditOrigin editOrigin = bdfUser.newEditOrigin(Domains.EXPORTATION + "/" + COMMANDNAME);
        TemplateStorage.Unit defaultTemplateStorageUnit = getDefaultTemplateStorageUnit();
        try {
            TemplateDescription templateDescription = bdfServer.getTransformationManager().createTemplate(defaultTemplateStorageUnit, editOrigin);
            putResultObject(TEMPLATEDESCRIPTION_OBJ, templateDescription);
            setDone("_ done.exportation.templatecreation", templateKey.toString());
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }

    }

    @Override
    protected boolean checkParameters() throws BdfInstructionException {
        BdfInstructionUtils.checkGlobalAdminPermission(bdfServer, bdfUser);
        TransformationDescription transformationDescription = BdfInstructionUtils.getMandatoryTransformationDescription(bdfServer, requestMap);
        TransformationKey transformationKey = transformationDescription.getTransformationKey();
        String templateName = getMandatory(NEW_TEMPLATE_PARAMNAME);
        templateName = templateName.trim();
        if (templateName.length() == 0) {
            setError("_ error.empty.templatename", templateName);
            return false;
        }
        if (!StringUtils.isTechnicalName(templateName, true)) {
            setError("_ error.wrong.templatename", templateName);
            return false;
        }
        withIf = requestMap.isTrue(WITHIF_PARAMNAME);
        withExtraction = requestMap.isTrue(WITHEXTRACTION_PARAMNAME);
        creationType = getMandatory(TYPE_PARAMNAME);
        ValidExtension validExtension = getValidExtension(creationType);
        templateType = getTemplateType(creationType);
        tableExportName = requestMap.getParameter(TABLEXPORT_PARAMNAME);
        if (tableExportName == null) {
            tableExportName = "";
        } else {
            tableExportName = tableExportName.trim();
        }
        try {
            templateKey = TemplateKey.parse(transformationKey, validExtension, templateName);
        } catch (ParseException pe) {
            throw new ShouldNotOccurException("Test done before");
        }
        if (bdfServer.getTransformationManager().containsTemplate(templateKey)) {
            setError("_ error.existing.template", templateName);
            return false;
        }
        if (transformationKey.isCorpusTransformationKey()) {
            Corpus corpus = (Corpus) bdfServer.getFichotheque().getSubset(transformationKey.toCorpusKey());
            if (corpus == null) {
                throw new BdfInstructionException(BdfInstructionUtils.unknownParameterValue(InteractionConstants.TRANSFORMATION_PARAMNAME, transformationKey.toCorpusKey().toString()));
            }
        }
        if (!TemplateCompilerUtils.isValidType(templateType, templateKey)) {
            setError("_ error.unknown.transformation.templatetype", creationType);
            return false;
        }
        return true;
    }

    private static ValidExtension getValidExtension(String type) throws BdfInstructionException {
        switch (type) {
            case HTML_XSLT_TYPE_PARAMVALUE:
                return null;
            case ODT_XSLT_TYPE_PARAMVALUE:
            case ODT_PROPERTIES_TYPE_PARAMVALUE:
                return ValidExtension.ODT;
            default:
                throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(TYPE_PARAMNAME, type));
        }
    }

    private static String getTemplateType(String type) throws BdfInstructionException {
        switch (type) {
            case HTML_XSLT_TYPE_PARAMVALUE:
            case ODT_XSLT_TYPE_PARAMVALUE:
                return "xslt";
            case ODT_PROPERTIES_TYPE_PARAMVALUE:
                return "properties";
            default:
                throw new BdfInstructionException(BdfInstructionUtils.wrongParameterValue(TYPE_PARAMNAME, type));
        }
    }

    private TemplateStorage.Unit getDefaultTemplateStorageUnit() {
        switch (creationType) {
            case HTML_XSLT_TYPE_PARAMVALUE:
                return TemplateStorageUnitFactory.newXsltHtml(bdfServer, templateKey, withIf, withExtraction);
            case ODT_XSLT_TYPE_PARAMVALUE:
                return TemplateStorageUnitFactory.newXsltOdt(bdfServer, templateKey, withExtraction);
            case ODT_PROPERTIES_TYPE_PARAMVALUE:
                return TemplateStorageUnitFactory.newPropertiesOdt(bdfServer, templateKey, tableExportName);
            default:
                throw new ShouldNotOccurException("test done before");
        }
    }

}
