/* BdfServer_Get - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.streamproducers;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.exportation.transformation.TransformationConstants;
import fr.exemole.bdfserver.api.managers.TransformationManager;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.tools.exportation.table.BdfTableExportUtils;
import fr.exemole.bdfserver.tools.exportation.table.CellEngineProviderFactory;
import fr.exemole.bdfserver.tools.exportation.table.CellEngineUtils;
import fr.exemole.bdfserver.tools.exportation.transformation.BdfTransformationUtils;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.io.IOException;
import java.io.OutputStream;
import java.io.StringReader;
import java.text.ParseException;
import java.util.Date;
import java.util.Map;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.exportation.table.Cell;
import net.fichotheque.exportation.table.CellEngine;
import net.fichotheque.exportation.table.CellEngineProvider;
import net.fichotheque.exportation.table.SubsetTable;
import net.fichotheque.exportation.transformation.SimpleTemplate;
import net.fichotheque.exportation.transformation.StreamTemplate;
import net.fichotheque.exportation.transformation.TemplateKey;
import net.fichotheque.exportation.transformation.TransformationKey;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.ExtractionSource;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.tools.extraction.builders.BuilderUtils;
import net.fichotheque.xml.extraction.ExtractionXMLPart;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.io.StreamProducer;
import net.mapeadores.util.servlets.HttpAccessException;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.ServletUtils;
import net.mapeadores.util.text.ValidExtension;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XmlProducer;


/**
 *
 * @author Vincent Calame
 */
public final class ExtractionStream {

    private ExtractionStream() {
    }

    public static ResponseHandler toStreamTemplateResponseHandler(BdfServer bdfServer, BdfUser bdfUser, ExtractionContext extractionContext, TransformationKey transformationKey, ValidExtension validExtension, String templateName, Object dynamicObject) {
        if (validExtension == null) {
            throw new IllegalArgumentException("validExtension is null");
        }
        StreamTemplate streamTemplate = getStreamTemplate(bdfServer, bdfUser, extractionContext, transformationKey, validExtension, templateName);
        StreamProducer streamProducer;
        if (streamTemplate instanceof StreamTemplate.Xslt) {
            streamProducer = getXsltStreamProducer(bdfServer, bdfUser, extractionContext, transformationKey, dynamicObject, (StreamTemplate.Xslt) streamTemplate);
        } else if (streamTemplate instanceof StreamTemplate.Properties) {
            streamProducer = getPropertiesStreamProducer(bdfServer, bdfUser, extractionContext, dynamicObject, (StreamTemplate.Properties) streamTemplate);
        } else {
            throw new HttpAccessException("_ error.unsupported.unknownimplementation", streamTemplate.getClass().getCanonicalName());
        }
        return ServletUtils.wrap(streamProducer);
    }

    private static StreamProducer getXsltStreamProducer(BdfServer bdfServer, BdfUser bdfUser, ExtractionContext extractionContext, TransformationKey transformationKey, Object dynamicObject, StreamTemplate.Xslt xslStreamTemplate) {
        ExtractionDef extractionDef = xslStreamTemplate.getCustomExtractionDef();
        if (extractionDef == null) {
            extractionDef = BdfTransformationUtils.getDefaultExtractionDef(bdfServer, transformationKey);
        }
        ExtractionSource extractionSource = BuilderUtils.getExtractionSource(dynamicObject, extractionContext, extractionDef);
        String extractionString = ExtractionXMLPart.toString(extractionContext, extractionDef, extractionSource, false);
        return new XsltStreamProducer(null, extractionString, xslStreamTemplate, BdfTransformationUtils.newParamMap(bdfServer, bdfUser));
    }

    private static StreamProducer getPropertiesStreamProducer(BdfServer bdfServer, BdfUser bdfUser, ExtractionContext extractionContext, Object dynamicObject, StreamTemplate.Properties streamTemplate) {
        if ((dynamicObject == null) || (!(dynamicObject instanceof FicheMeta))) {
            throw new HttpAccessException("_ error.unsupported.ficheonlytemplate");
        }
        String tableExportName = streamTemplate.getTableExportName();
        FicheMeta ficheMeta = (FicheMeta) dynamicObject;
        CellEngine cellEngine;
        if (tableExportName.isEmpty()) {
            SubsetTable subsetTable = BdfTableExportUtils.toDefaultSubsetTable(bdfServer, ficheMeta.getCorpus(), BdfTableExportUtils.ALL_FICHETABLEPARAMETERS, bdfUser.getPermissionSummary(bdfServer));
            cellEngine = CellEngineUtils.newCellEngine(bdfServer, extractionContext, null, subsetTable);
        } else {
            CellEngineProvider cellEngineProvider = CellEngineProviderFactory.newInstance(bdfServer, extractionContext, false);
            cellEngine = cellEngineProvider.getCellEngine(tableExportName);
            if (cellEngine == null) {
                throw new HttpAccessException("_ error.unknown.tablexport", tableExportName);
            }
        }
        Cell[] cellArray = cellEngine.getCellArray(ficheMeta);
        if (cellArray == null) {
            throw new HttpAccessException("_ error.empty.subsettable", tableExportName, ficheMeta.getCorpus().getSubsetKey());
        }
        return new PropertiesStreamProducer(null, cellArray, streamTemplate);
    }

    private static StreamTemplate getStreamTemplate(BdfServer bdfServer, BdfUser bdfUser, ExtractionContext extractionContext, TransformationKey transformationKey, ValidExtension validExtension, String templateName) {
        TemplateKey templateKey;
        boolean userTemplate;
        if ((templateName != null) && (templateName.length() > 0)) {
            userTemplate = false;
            try {
                templateKey = TemplateKey.parse(transformationKey, validExtension, templateName);
            } catch (ParseException pe) {
                throw new HttpAccessException("_ error.wrong.templatename", templateName);
            }
        } else {
            userTemplate = true;
            templateKey = BdfUserUtils.getStreamTemplateKey(bdfUser, transformationKey, validExtension);
        }
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        StreamTemplate streamTemplate = transformationManager.getStreamTemplate(templateKey);
        if (streamTemplate == null) {
            if (userTemplate) {
                streamTemplate = transformationManager.getStreamTemplate(templateKey.toDefaultTemplateKey());
                if (streamTemplate == null) {
                    throw new HttpAccessException("_ error.unsupported.defaulttemplate", validExtension.toString(), transformationKey.toString());
                }
            } else {
                throw new HttpAccessException("_ error.unknown.template", templateName);
            }
        }
        return streamTemplate;
    }

    public static ResponseHandler toXmlResponseHandler(BdfServer bdfServer, BdfUser bdfUser, ExtractionContext extractionContext, TransformationKey transformationKey, String templateName, Object dynamicObject) {
        TemplateKey templateKey;
        boolean userTemplate;
        if ((templateName != null) && (templateName.length() > 0)) {
            userTemplate = false;
            try {
                templateKey = TemplateKey.parse(transformationKey.toString() + "/" + templateName);
            } catch (ParseException pe) {
                throw new HttpAccessException("_ error.wrong.templatename", templateName);
            }
        } else {
            userTemplate = true;
            templateKey = BdfUserUtils.getSimpleTemplateKey(bdfUser, transformationKey);
        }
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        ExtractionDef extractionDef;
        if (templateKey.isSimpleTemplate()) {
            SimpleTemplate simpleTemplate = transformationManager.getSimpleTemplate(templateKey);
            if (simpleTemplate == null) {
                if (userTemplate) {
                    simpleTemplate = transformationManager.getSimpleTemplate(templateKey.toDefaultTemplateKey());
                    if (simpleTemplate == null) {
                        throw new HttpAccessException("_ error.unsupported.defaulttemplate", "", transformationKey.toString());
                    }
                } else {
                    throw new HttpAccessException("_ error.unknown.template", templateName);
                }
            }
            extractionDef = simpleTemplate.getCustomExtractionDef();
        } else {
            StreamTemplate streamTemplate = transformationManager.getStreamTemplate(templateKey);
            if (streamTemplate == null) {
                if (userTemplate) {
                    streamTemplate = transformationManager.getStreamTemplate(templateKey.toDefaultTemplateKey());
                    if (streamTemplate == null) {
                        throw new HttpAccessException("_ error.unsupported.defaulttemplate", templateKey.getExtension(), transformationKey.toString());
                    }
                } else {
                    throw new HttpAccessException("_ error.unknown.template", templateName);
                }
            }
            if (!(streamTemplate instanceof StreamTemplate.Xslt)) {
                throw new HttpAccessException("_ error.unsupported.noextractiontemplate", templateName);
            }
            extractionDef = ((StreamTemplate.Xslt) streamTemplate).getCustomExtractionDef();
        }
        if (extractionDef == null) {
            extractionDef = BdfTransformationUtils.getDefaultExtractionDef(bdfServer, transformationKey);
            if (extractionDef == null) {
                throw new HttpAccessException("_ error.unsupported.defaultextractiondef", transformationKey.toString());
            }
        }
        return toXmlResponseHandler(bdfServer, bdfUser, extractionContext, extractionDef, dynamicObject);
    }

    public static ResponseHandler toXmlResponseHandler(BdfServer bdfServer, BdfUser bdfUser, ExtractionContext extractionContext, ExtractionDef extractionDef, Object dynamicObject) {
        ExtractionSource extractionSource = BuilderUtils.getExtractionSource(dynamicObject, extractionContext, extractionDef);
        return ServletUtils.wrap(new ExtractionXmlProducer(extractionContext, extractionDef, extractionSource));
    }

    public static ResponseHandler toSimpleTemplateResponseHandler(BdfServer bdfServer, BdfUser bdfUser, ExtractionContext extractionContext, TransformationKey transformationKey, String templateName, Object dynamicObject) {
        TemplateKey templateKey;
        boolean userTemplate;
        if ((templateName != null) && (templateName.length() > 0)) {
            userTemplate = false;
            try {
                templateKey = TemplateKey.parse(transformationKey, templateName);
            } catch (ParseException pe) {
                throw new HttpAccessException("_ error.wrong.templatename", templateName);
            }
        } else {
            userTemplate = true;
            templateKey = BdfUserUtils.getSimpleTemplateKey(bdfUser, transformationKey);
        }
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        SimpleTemplate simpleTemplate = transformationManager.getSimpleTemplate(templateKey);
        if (simpleTemplate == null) {
            if (userTemplate) {
                simpleTemplate = transformationManager.getSimpleTemplate(templateKey.toDefaultTemplateKey());
                if (simpleTemplate == null) {
                    throw new HttpAccessException("_ error.unsupported.defaulttemplate", "", transformationKey.toString());
                }
            } else {
                throw new HttpAccessException("_ error.unknown.template", templateName);
            }
        }
        ExtractionDef extractionDef = simpleTemplate.getCustomExtractionDef();
        if (extractionDef == null) {
            extractionDef = BdfTransformationUtils.getDefaultExtractionDef(bdfServer, transformationKey);
        }
        ExtractionSource extractionSource = BuilderUtils.getExtractionSource(dynamicObject, extractionContext, extractionDef);
        String extractionString = ExtractionXMLPart.toString(extractionContext, extractionDef, extractionSource, false);
        Map<String, Object> paramMap = BdfTransformationUtils.newParamMap(bdfServer, bdfUser);
        paramMap.put(TransformationConstants.BDF_FICHOTHEQUEPATH_PARAMETER, "../");
        paramMap.put(TransformationConstants.EXTERNALTARGET_PARAMETER, "_blank");
        StreamProducer streamProducer = new SimpleTemplateStreamProducer(null, extractionString, simpleTemplate, paramMap);
        return ServletUtils.wrap(streamProducer);
    }

    public static ResponseHandler toSimpleTemplateResponseHandler(BdfServer bdfServer, BdfUser bdfUser, String sourceString, TransformationKey transformationKey) {
        TemplateKey templateKey = BdfUserUtils.getSimpleTemplateKey(bdfUser, transformationKey);
        TransformationManager transformationManager = bdfServer.getTransformationManager();
        SimpleTemplate simpleTemplate = transformationManager.getSimpleTemplate(templateKey);
        if ((simpleTemplate == null) && (!templateKey.isDefault())) {
            simpleTemplate = transformationManager.getSimpleTemplate(templateKey.toDefaultTemplateKey());
        }
        Map<String, Object> paramMap = BdfTransformationUtils.newParamMap(bdfServer, bdfUser);
        paramMap.put(TransformationConstants.BDF_FICHOTHEQUEPATH_PARAMETER, "../");
        paramMap.put(TransformationConstants.EXTERNALTARGET_PARAMETER, "_blank");
        StreamProducer streamProducer = new SimpleTemplateStreamProducer(null, sourceString, simpleTemplate, paramMap);
        return ServletUtils.wrap(streamProducer);
    }


    private static class SimpleTemplateStreamProducer implements StreamProducer {

        private final String fileName;
        private final String extractionString;
        private final SimpleTemplate simpleTemplate;
        private final Map<String, Object> paramMap;

        private SimpleTemplateStreamProducer(String fileName, String extractionString, SimpleTemplate simpleTemplate, Map<String, Object> paramMap) {
            this.fileName = fileName;
            this.extractionString = extractionString;
            this.simpleTemplate = simpleTemplate;
            this.paramMap = paramMap;
        }

        @Override
        public String getMimeType() {
            return simpleTemplate.getMimeType();
        }

        @Override
        public String getCharset() {
            return simpleTemplate.getCharset();
        }

        @Override
        public String getFileName() {
            return fileName;
        }

        @Override
        public void writeStream(OutputStream outputStream) throws IOException {
            String result = simpleTemplate.transform(extractionString, paramMap);
            IOUtils.copy(new StringReader(result), outputStream, simpleTemplate.getCharset());
        }

    }


    private static class XsltStreamProducer implements StreamProducer {

        private final String fileName;
        private final String extractionString;
        private final StreamTemplate.Xslt streamTemplate;
        private final Map<String, Object> paramMap;

        private XsltStreamProducer(String fileName, String extractionString, StreamTemplate.Xslt streamTemplate, Map<String, Object> paramMap) {
            this.fileName = fileName;
            this.extractionString = extractionString;
            this.streamTemplate = streamTemplate;
            this.paramMap = paramMap;
        }

        @Override
        public String getMimeType() {
            return streamTemplate.getMimeType();
        }

        @Override
        public String getCharset() {
            return null;
        }

        @Override
        public String getFileName() {
            return fileName;
        }

        @Override
        public void writeStream(OutputStream outputStream) throws IOException {
            streamTemplate.transform(extractionString, paramMap, outputStream);
        }

    }


    private static class PropertiesStreamProducer implements StreamProducer {

        private final String fileName;
        private final Cell[] cellArray;
        private final StreamTemplate.Properties streamTemplate;

        private PropertiesStreamProducer(String fileName, Cell[] cellArray, StreamTemplate.Properties streamTemplate) {
            this.fileName = fileName;
            this.cellArray = cellArray;
            this.streamTemplate = streamTemplate;
        }

        @Override
        public String getMimeType() {
            return streamTemplate.getMimeType();
        }

        @Override
        public String getCharset() {
            return null;
        }

        @Override
        public String getFileName() {
            return fileName;
        }

        @Override
        public void writeStream(OutputStream outputStream) throws IOException {
            streamTemplate.fill(cellArray, outputStream);
        }

    }


    private static class ExtractionXmlProducer implements XmlProducer {

        private final ExtractionContext extractionContext;
        private final ExtractionDef extractionDef;
        private final ExtractionSource extractionSource;

        private ExtractionXmlProducer(ExtractionContext extractionContext, ExtractionDef extractionDef, ExtractionSource extractionSource) {
            this.extractionContext = extractionContext;
            this.extractionDef = extractionDef;
            this.extractionSource = extractionSource;
        }

        @Override
        public void writeXml(Appendable appendable) throws IOException {
            AppendableXMLWriter xmlWriter = XMLUtils.toXMLWriter(appendable, 0);
            xmlWriter.appendXMLDeclaration();
            ExtractionXMLPart extractionXMLPart = new ExtractionXMLPart(xmlWriter, extractionContext);
            extractionXMLPart.addExtraction(extractionDef, new Date(), extractionSource, true);
        }

    }

}
