/* BdfServer_Html - Copyright (c) 2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html;

import fr.exemole.bdfserver.api.BdfExtensionReference;
import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.ProducerParameters;
import fr.exemole.bdfserver.api.storage.ResourceFolder;
import fr.exemole.bdfserver.api.storage.ResourceStorage;
import fr.exemole.bdfserver.html.jslib.JsLib;
import fr.exemole.bdfserver.html.jslib.JsLibCatalog;
import fr.exemole.bdfserver.html.jslib.JsLibUtils;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.apps.AppConf;
import fr.exemole.bdfserver.tools.storage.StorageUtils;
import net.mapeadores.util.html.JsObject;
import net.mapeadores.util.html.TrustedHtml;
import net.mapeadores.util.html.jsoup.TrustedHtmlFactories;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.localisation.LangPreference;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public class AppHtmlProducer extends BdfServerHtmlProducer {

    private final AppConf appConf;

    public AppHtmlProducer(BdfServer bdfServer, LangPreference langPreference, AppConf appConf) {
        super(bdfServer, langPreference);
        this.appConf = appConf;
        init();
    }

    public AppHtmlProducer(ProducerParameters producerParameters, AppConf appConf) {
        super(producerParameters);
        this.appConf = appConf;
        init();
    }

    private void init() {
        setWithJavascript(true);
        setWithIconScan(true);
        if (appConf.getBoolean(AppConf.CORE_INCLUDEFICHECSS)) {
            addFicheCss();
        }
        for (String jslibName : appConf.getArray(AppConf.CORE_JSLIBS)) {
            int idx = jslibName.indexOf(":");
            if (idx == -1) {
                JsLib jsLib = JsLibCatalog.CORE.getJsLib(jslibName);
                if (jsLib != null) {
                    addJsLib(jsLib);
                } else {
                    addLog("[core_jslibs] Unknown jsLib: " + jslibName);
                }
            } else {
                String extensionName = jslibName.substring(0, idx);
                BdfExtensionReference reference = bdfServer.getExtensionManager().getBdfExtensionReference(extensionName);
                if (reference != null) {
                    JsLibProvider jsLibProvider = (JsLibProvider) reference.getImplementation(JsLibProvider.class);
                    if (jsLibProvider != null) {
                        JsLib jsLib = jsLibProvider.getJsLib(jslibName.substring(idx + 1));
                        if (jsLib != null) {
                            addJsLib(jsLib);
                        } else {
                            addLog("[core_jslibs] Unknown extension jslib: " + jslibName);
                        }
                    } else {
                        addLog("[core_jslibs] extension has no jslib provider: " + extensionName);
                    }
                } else {
                    addLog("[core_jslibs] Unknown extension: " + extensionName);
                }
            }
        }
        addJsLib(JsLibUtils.getAppJsLib(bdfServer, appConf));
        addThemeCss(appConf.getArray(AppConf.CORE_THEMECSSFILES));
        populateAppThemeCss(appConf.getAppName());
        populateSupplementaryCss(appConf.getAppName());
        String bodyCssClass = appConf.getString(AppConf.CORE_BODYCSSCLASS);
        if (bodyCssClass != null) {
            setBodyCssClass(bodyCssClass);
        }
        populateCustomCss(appConf.getAppName());
    }

    @Override
    public void printHtml() {
        start();
        RelativePath appHtmlPath = StorageUtils.buildAppResourcePath(appConf.getAppName(), "app.html");
        DocStream docStream = StorageUtils.getResourceDocStream(bdfServer, appHtmlPath);
        if (docStream != null) {
            String appHtml = BdfServerUtils.toString(docStream);
            TrustedHtml trustedHtml = TrustedHtmlFactories.WELLFORMED.build(appHtml);
            this
                    .__append(trustedHtml);
        }
        JsObject jsObject = appConf.getArgsJsObject();
        if (jsObject != null) {
            this
                    .SCRIPT()
                    .__jsObject("ARGS", jsObject)
                    ._SCRIPT();
        }
        end();
    }

    private void populateAppThemeCss(String appName) {
        RelativePath themeCssFolderPath = StorageUtils.buildAppResourcePath(appName, "theme/css");
        for (ResourceStorage resourceStorage : bdfServer.getResourceStorageList()) {
            ResourceFolder cssFolder = resourceStorage.getResourceFolder(themeCssFolderPath);
            if (cssFolder != null) {
                for (String resourceName : cssFolder.getResourceNameList()) {
                    if (resourceName.endsWith(".css")) {
                        addAppThemeCss(appName, resourceName);
                    }
                }
            }
        }
    }

    private void populateSupplementaryCss(String appName) {
        RelativePath cssFolderPath = StorageUtils.buildAppResourcePath(appName, "css");
        for (ResourceStorage resourceStorage : bdfServer.getResourceStorageList()) {
            ResourceFolder cssFolder = resourceStorage.getResourceFolder(cssFolderPath);
            if (cssFolder != null) {
                for (String resourceName : cssFolder.getResourceNameList()) {
                    if (resourceName.endsWith(".css")) {
                        addSupplementaryCss(cssFolderPath.buildChild(resourceName));
                    }
                }
            }
        }
    }

    private void populateCustomCss(String appName) {
        RelativePath appCssPath = RelativePath.build("custom/app-" + appName + ".css");
        if (StorageUtils.containsResource(bdfServer, appCssPath)) {
            addCustomCss(appCssPath);
        }
    }

}
