/* BdfServer_Html - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.html.forms;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.subsettree.SubsetTree;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.html.consumers.SelectOption;
import fr.exemole.bdfserver.html.consumers.SubsetTreeOptions;
import fr.exemole.bdfserver.html.consumers.attributes.Appelant;
import fr.exemole.bdfserver.html.consumers.attributes.Deploy;
import fr.exemole.bdfserver.tools.interaction.FicheQueryParser;
import fr.exemole.bdfserver.tools.subsettree.TreeFilterEngine;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import net.fichotheque.Fichotheque;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FieldQuery;
import net.fichotheque.selection.MotcleCondition;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.selection.PeriodCondition;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.FichothequeUtils;
import net.fichotheque.utils.SelectionUtils;
import net.fichotheque.utils.SphereUtils;
import net.mapeadores.util.conditions.Condition;
import net.mapeadores.util.conditions.ConditionsConstants;
import net.mapeadores.util.conditions.ConditionsUtils;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.RangeUtils;
import net.mapeadores.util.primitives.Ranges;


/**
 *
 * @author Vincent Calame
 */
public class SelectionFormPrinter {

    private final static SelectOption[] DISCARDFILTER_OPTIONS = {
        SelectOption.init(FicheQuery.ALL_DISCARDFILTER).textLocKey("_ label.selection.discardfilter_all"),
        SelectOption.init(FicheQuery.NONE_DISCARDFILTER).textLocKey("_ label.selection.discardfilter_none"),
        SelectOption.init(FicheQuery.ONLY_DISCARDFILTER).textLocKey("_ label.selection.discardfilter_only")
    };
    private final HtmlPrinter htmlPrinter;
    private final Fichotheque fichotheque;
    private final BdfServer bdfServer;
    private final Lang workingLang;
    private final BdfUser bdfUser;
    private final PermissionSummary permissionSummary;
    private FicheQuery ficheQuery;

    public SelectionFormPrinter(FicheQuery ficheQuery, BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, HtmlPrinter htmlPrinter) {
        this.ficheQuery = ficheQuery;
        this.htmlPrinter = htmlPrinter;
        this.fichotheque = bdfServer.getFichotheque();
        this.bdfUser = bdfUser;
        this.permissionSummary = permissionSummary;
        this.workingLang = bdfUser.getWorkingLang();
        this.bdfServer = bdfServer;
    }

    public boolean printFieldsets() {
        htmlPrinter
                .DIV("selectform-Col3")
                .__(printIdFieldset())
                .__(printCorpusFieldset())
                .__(printDiscardFilterFieldset())
                ._DIV()
                .DIV("selectform-Col3")
                .__(printFieldConditionFieldset())
                .__(printRedacteursFieldset())
                .__(printPeriodFieldSet())
                ._DIV()
                .DIV("selectform-Col3")
                .__(printMotcleConditionFieldsets())
                ._DIV();
        return true;
    }

    public static SelectionFormPrinter init(FicheQuery ficheQuery, BdfServer bdfServer, BdfUser bdfUser, PermissionSummary permissionSummary, HtmlPrinter htmlPrinter) {
        return new SelectionFormPrinter(ficheQuery, bdfServer, bdfUser, permissionSummary, htmlPrinter);
    }

    public static boolean printSeparator(HtmlPrinter hp) {
        hp
                .HR("selectform-Separator");
        return true;
    }

    private boolean printIdFieldset() {
        String idValue = "";
        Ranges idRanges = ficheQuery.getIdRanges();
        if (idRanges != null) {
            idValue = RangeUtils.positiveRangetoString(idRanges);
        }
        htmlPrinter
                .FIELDSET("selectform-Id")
                .LEGEND()
                .__localize("_ label.selection.id")
                .__colon()
                ._LEGEND()
                .TEXTAREA(HA.name(FicheQueryParser.ID_PARAMNAME).classes("selectform-Input").rows(3))
                .__escape(idValue, true)
                ._TEXTAREA()
                ._FIELDSET();
        return true;
    }

    private boolean printFieldConditionFieldset() {
        String fieldValue = "";
        FieldQuery fieldQuery = ficheQuery.getFieldQuery();
        short currentScope = FieldQuery.TITRE_SCOPE;
        if (fieldQuery != null) {
            fieldValue = ConditionsUtils.conditionToSimpleString(fieldQuery.getContentCondition());
            currentScope = fieldQuery.getScope();
        }
        htmlPrinter
                .FIELDSET("selectform-FieldCondition")
                .LEGEND()
                .__localize("_ label.selection.fieldcondition")
                .__colon()
                ._LEGEND()
                .INPUT_text(HA.name(FicheQueryParser.FIELDCONDITION_PARAMNAME).value(fieldValue).classes("selectform-Input"))
                .BR()
                .__(addScopeRadio(FieldQuery.TITRE_SCOPE, "_ label.selection.scope_titre", currentScope))
                .__(addScopeRadio(FieldQuery.ENTETE_SCOPE, "_ label.selection.scope_entete", currentScope))
                .__(addScopeRadio(FieldQuery.FICHE_SCOPE, "_ label.selection.scope_fiche", currentScope))
                .__(addScopeRadio(FieldQuery.SELECTION_SCOPE, "_ label.selection.scope_selection", currentScope))
                ._FIELDSET();
        return true;
    }

    private boolean addScopeRadio(short scope, String key, short current) {
        String appelantFieldSelection = htmlPrinter.generateId();
        String appelantFieldSelectionAnchorId = htmlPrinter.generateId();
        String fieldSelectionAreaId = htmlPrinter.generateId();
        String fieldSelectionAreaClasses = "";
        if ((htmlPrinter.isWithJavascript()) && (current != scope)) {
            fieldSelectionAreaClasses = "hidden";
        }
        HtmlAttributes scopeAttr = HA.name(FicheQueryParser.FIELDCONDITION_SCOPE_PARAMNAME).value(SelectionUtils.fieldConditionToString(scope)).checked((scope == current));
        if (scope == FieldQuery.SELECTION_SCOPE) {
            scopeAttr.populate(Deploy.radio(fieldSelectionAreaId));
        }
        htmlPrinter
                .LABEL("selectform-Label")
                .INPUT_radio(scopeAttr)
                .__localize(key)
                ._LABEL();
        if (scope == FieldQuery.SELECTION_SCOPE) {
            String fieldSelection = "";
            FieldQuery fieldQuery = ficheQuery.getFieldQuery();
            if (fieldQuery != null) {
                fieldSelection = toString(fieldQuery.getFieldKeyList());
            }
            htmlPrinter
                    .DIV(HA.id(fieldSelectionAreaId).classes(fieldSelectionAreaClasses))
                    .INPUT_text(HA
                            .id(appelantFieldSelection)
                            .name(FicheQueryParser.FIELDCONDITION_FIELDSELECTION_PARAMNAME)
                            .value(fieldSelection)
                            .classes("selectform-Input")
                            .populate(Appelant.field().anchor(appelantFieldSelectionAnchorId)))
                    .DIV("selectform-AndOr")
                    .SPAN(HA.id(appelantFieldSelectionAnchorId))
                    .__space()
                    ._SPAN()
                    ._DIV()
                    ._DIV();
        }
        return true;
    }

    private boolean printCorpusFieldset() {
        String detailId = htmlPrinter.generateId();
        List<SubsetKey> corpusKeyList = ficheQuery.getCorpusKeyList();
        boolean empty = corpusKeyList.isEmpty();
        HashSet<SubsetKey> selectedSet = new HashSet<SubsetKey>();
        if (!empty) {
            selectedSet.addAll(corpusKeyList);
        }
        SubsetTree filteredSubsetTree = TreeFilterEngine.read(permissionSummary, bdfServer.getTreeManager().getSubsetTree(SubsetKey.CATEGORY_CORPUS));
        htmlPrinter
                .FIELDSET("selectform-Corpus")
                .LEGEND()
                .__localize("_ label.selection.corpus")
                .__colon()
                ._LEGEND()
                .__(addRadios(FicheQueryParser.CORPUS_ALL_PARAMNAME, detailId, empty, "_ label.selection.corpus_all"))
                .DIV(getDetailAttributes(detailId, empty))
                .__(SubsetTreeOptions.init(filteredSubsetTree, bdfServer, workingLang)
                        .onlyNames(true)
                        .withKeys(false)
                        .selectedSet(selectedSet)
                        .checkboxMode(HA.name(FicheQueryParser.CORPUS_SELECTION_PARAMNAME), HA.classes("selectform-Label"), HA.classes("selectform-GroupTitle"), HA.classes("selectform-GroupDiv"))
                )
                ._DIV()
                ._FIELDSET();
        return true;
    }

    private boolean printDiscardFilterFieldset() {
        String currentDiscardFilter = ficheQuery.getDiscardFilter();
        htmlPrinter
                .FIELDSET("selectform-DiscardFilter")
                .LEGEND()
                .__localize("_ label.selection.discardfilter")
                .__colon()
                ._LEGEND()
                .SELECT(FicheQueryParser.DISCARDFILTER_PARAMNAME);
        for (SelectOption option : DISCARDFILTER_OPTIONS) {
            htmlPrinter
                    .__(option, currentDiscardFilter);
        }
        htmlPrinter
                ._SELECT()
                ._FIELDSET();
        return true;
    }

    private boolean printRedacteursFieldset() {
        String appelantRedacteurId = htmlPrinter.generateId();
        String appelantRedacteursAnchorId = htmlPrinter.generateId();
        String detailId = htmlPrinter.generateId();
        String redacteurFilter = ficheQuery.getRedacteurFilter();
        boolean all = (redacteurFilter.equals(FicheQuery.ALL_REDACTEURFILTER));
        String redacteursString = "";
        switch (redacteurFilter) {
            case FicheQuery.ANY_REDACTEURFILTER:
                redacteursString = "*";
                break;
            case FicheQuery.NONE_REDACTEURFILTER:
                redacteursString = "!*";
                break;
            case FicheQuery.SOME_REDACTEURFILTER:
                redacteursString = toRedacteursString();
                break;
        }
        htmlPrinter
                .FIELDSET("selectform-Redacteurs")
                .LEGEND()
                .__localize("_ label.selection.redacteurs")
                .__colon()
                ._LEGEND()
                .__(addRadios(FicheQueryParser.REDACTEURS_ALL_PARAMNAME, detailId, all, "_ label.selection.redacteurs_all"))
                .DIV(getDetailAttributes(detailId, all))
                .INPUT_text(HA
                        .id(appelantRedacteurId)
                        .name(FicheQueryParser.REDACTEURS_SELECTION_PARAMNAME)
                        .value(redacteursString)
                        .classes("selectform-Input")
                        .populate(Appelant.redacteur().sphere(bdfUser).anchor(appelantRedacteursAnchorId)))
                .DIV("selectform-AndOr")
                .SPAN(HA.id(appelantRedacteursAnchorId))
                .__space()
                ._SPAN()
                ._DIV()
                ._DIV()
                ._FIELDSET();
        return true;
    }

    private boolean printPeriodFieldSet() {
        String periodFieldListAreaId = htmlPrinter.generateId();
        String appelantPeriodFieldListId = htmlPrinter.generateId();
        String appelantPeriodFieldListAnchorId = htmlPrinter.generateId();
        String startDate = "";
        String endDate = "";
        boolean onCreationDate = true;
        boolean onModificationDate = false;
        boolean withFieldSelection = false;
        String fieldSelection = "";
        PeriodCondition periodCondition = ficheQuery.getPeriodCondition();
        if (periodCondition != null) {
            switch (periodCondition.getStartType()) {
                case PeriodCondition.ANY_TYPE:
                    startDate = PeriodCondition.ANY_CHAR;
                    break;
                case PeriodCondition.DATE_TYPE:
                    startDate = periodCondition.getStartDate().toString();
                    break;
            }
            switch (periodCondition.getEndType()) {
                case PeriodCondition.ANY_TYPE:
                    endDate = PeriodCondition.ANY_CHAR;
                    break;
                case PeriodCondition.DATE_TYPE:
                    endDate = periodCondition.getEndDate().toString();
                    break;
            }
            onCreationDate = periodCondition.isOnCreationDate();
            onModificationDate = periodCondition.isOnModificationDate();
            fieldSelection = toString(periodCondition.getFieldKeyList());
            withFieldSelection = !fieldSelection.isEmpty();
        }
        String periodFieldListAreaClasses = "";
        if ((htmlPrinter.isWithJavascript()) && (!withFieldSelection)) {
            periodFieldListAreaClasses = "hidden";
        }
        htmlPrinter
                .FIELDSET("selectform-Period")
                .LEGEND()
                .__localize("_ label.selection.period")
                .__colon()
                ._LEGEND()
                .LABEL("selectform-Label")
                .__localize("_ label.selection.period_start")
                .__colon()
                .INPUT_text(HA.name(FicheQueryParser.PERIOD_START_PARAMNAME).value(startDate).size("10").classes("selectform-Input").attr("data-selectform-role", "date"))
                ._LABEL()
                .LABEL("selectform-Label")
                .__localize("_ label.selection.period_end")
                .__colon()
                .INPUT_text(HA.name(FicheQueryParser.PERIOD_END_PARAMNAME).value(endDate).size("10").classes("selectform-Input").attr("data-selectform-role", "date"))
                ._LABEL()
                .LABEL("selectform-Label")
                .INPUT_checkbox(HA.name(FicheQueryParser.PERIOD_SCOPE_PARAMNAME).value(FicheQueryParser.CREATION_PARAMVALUE).checked(onCreationDate))
                .__space()
                .__localize("_ label.selection.period_onchrono_creation")
                ._LABEL()
                .LABEL("selectform-Label")
                .INPUT_checkbox(HA.name(FicheQueryParser.PERIOD_SCOPE_PARAMNAME).value(FicheQueryParser.MODIFICATION_PARAMVALUE).checked(onModificationDate))
                .__space()
                .__localize("_ label.selection.period_onchrono_modification")
                ._LABEL()
                .LABEL("selectform-Label")
                .INPUT_checkbox(HA.name(FicheQueryParser.PERIOD_SCOPE_PARAMNAME).value(FicheQueryParser.FIELDLIST_PARAMVALUE).checked(withFieldSelection).populate(Deploy.checkbox(periodFieldListAreaId)))
                .__space()
                .__localize("_ label.selection.period_fieldlist")
                ._LABEL()
                .DIV(HA.id(periodFieldListAreaId).classes(periodFieldListAreaClasses))
                .INPUT_text(HA.id(appelantPeriodFieldListId)
                        .name(FicheQueryParser.PERIOD_FIELDLIST_PARAMNAME)
                        .value(fieldSelection)
                        .classes("selectform-Input")
                        .populate(Appelant.field().fieldtype_datation().anchor(appelantPeriodFieldListAnchorId)))
                .DIV("selectform-AndOr")
                .SPAN(HA.id(appelantPeriodFieldListAnchorId))
                .__space()
                ._SPAN()
                ._DIV()
                ._DIV()
                ._FIELDSET();
        return true;
    }

    private String toRedacteursString() {
        List<String> redacteurGlobalIdList = ficheQuery.getRedacteurGlobalIdList();
        List<SubsetKey> sphereKeyList = ficheQuery.getSphereKeyList();
        SubsetKey defaultSphereKey = bdfUser.getRedacteur().getSphere().getSubsetKey();
        StringBuilder buf = new StringBuilder();
        for (String redacteurGlobalId : redacteurGlobalIdList) {
            Redacteur redacteur = SphereUtils.getRedacteur(fichotheque, redacteurGlobalId);
            if (redacteur != null) {
                if (!defaultSphereKey.equals(redacteur.getSphere().getSubsetKey())) {
                    buf.append(SphereUtils.toBracketStyle(redacteur));
                } else {
                    buf.append(redacteur.getLogin());
                }
                buf.append("; ");
            }
        }
        for (SubsetKey sphereKey : sphereKeyList) {
            buf.append('[');
            buf.append(sphereKey.getSubsetName());
            buf.append(']');
            buf.append("; ");
        }
        return buf.toString();
    }

    private String toString(List<FieldKey> fieldKeyList) {
        if (fieldKeyList.isEmpty()) {
            return "";
        }
        StringBuilder buf = new StringBuilder();
        for (FieldKey fieldKey : fieldKeyList) {
            if (buf.length() > 0) {
                buf.append(';');
            }
            buf.append(fieldKey.toString());
        }
        return buf.toString();
    }

    private boolean printMotcleConditionFieldsets() {
        short mode = ConditionsConstants.MODE_AND;
        boolean hidden = true;
        MotcleCondition motcleCondition = ficheQuery.getMotcleCondition();
        HtmlPrinter hp = htmlPrinter;
        StringBuilder buf = new StringBuilder();
        PrintWriter old = hp.initPrinter(buf);
        SelectionFormPrinter.this.printMotcleConditionFieldset(null, "nnn");
        hp.initPrinter(old);
        hp
                .DIV(HA.id("motcleFieldsets").attr("data-selectform-new-fieldset", buf.toString()));
        if (motcleCondition == null) {
            SelectionFormPrinter.this.printMotcleConditionFieldset(null, "1");
        } else {
            int number = 1;
            for (MotcleQuery motcleQuery : motcleCondition.getMotcleQueryList()) {
                SelectionFormPrinter.this.printMotcleConditionFieldset(motcleQuery, String.valueOf(number));
                number++;
            }
            mode = motcleCondition.getSelectMode();
            if (number > 2) {
                hidden = false;
            }
        }
        hp
                .__(printMotcleMode(mode, hidden))
                ._DIV();
        return true;
    }

    private boolean printMotcleMode(short mode, boolean hidden) {
        String modeClasses;
        if (hidden) {
            modeClasses = "selectform-MotcleMode hidden";
        } else {
            modeClasses = "selectform-MotcleMode";
        }
        htmlPrinter
                .DIV(HA.id("motcleMode").classes(modeClasses))
                .__localize("_ label.selection.motclemode")
                .__colon()
                .BR()
                .__(printAndOr(FicheQueryParser.MOTCLE_MODE_PARAMNAME, mode))
                ._DIV();
        return true;
    }

    private boolean printMotcleConditionFieldset(MotcleQuery motcleQuery, String suffix) {
        String anchorId = "motcleInputAnchor_" + suffix;
        HtmlPrinter hp = htmlPrinter;
        short typeMode = ConditionsConstants.MODE_AND;
        Condition motcleCondition = null;
        boolean withSelection = false;
        List<Subset> subsetList = null;
        if (motcleQuery != null) {
            motcleCondition = motcleQuery.getContentCondition();
            if (motcleCondition != null) {
                typeMode = motcleCondition.getMode();
            }
            List<SubsetKey> subsetKeyList = motcleQuery.getThesaurusKeyList();
            if (!subsetKeyList.isEmpty()) {
                withSelection = true;
                subsetList = new ArrayList<Subset>();
                for (SubsetKey thesaurusKey : subsetKeyList) {
                    Thesaurus thesaurus = (Thesaurus) fichotheque.getSubset(thesaurusKey);
                    if (thesaurus != null) {
                        subsetList.add(thesaurus);
                    }
                }
                if (subsetList.isEmpty()) {
                    subsetList = null;
                    withSelection = false;
                }
            }
        }
        boolean withSatellite = true;
        String detailId = "thesaurusList_" + suffix;
        HtmlAttributes att = HA.id("parMotcle_" + suffix).classes("selectform-MotcleCondition");
        hp
                .FIELDSET(att)
                .LEGEND()
                .__localize("_ label.selection.motclecondition", suffix)
                .__colon()
                ._LEGEND();
        hp
                .INPUT_hidden(FicheQueryParser.MOTCLE_SATELLITE_PREFIX + suffix, (withSatellite) ? "1" : "0");
        String value = (motcleCondition != null) ? ConditionsUtils.conditionToString(motcleCondition, false) : "";
        Appelant appelantData = Appelant.motcle().anchor(anchorId);
        if (subsetList != null) {
            for (Subset subset : subsetList) {
                appelantData.subsets(subset.getSubsetKey());
            }
        }
        hp
                .INPUT_text(HA.id("motcleInput_" + suffix)
                        .name(FicheQueryParser.MOTCLE_INPUT_PREFIX + suffix)
                        .value(value)
                        .classes("selectform-Input")
                        .populate(appelantData))
                .DIV("selectform-AndOr")
                .__(printAndOr(FicheQueryParser.MOTCLE_MODE_PREFIX + suffix, typeMode))
                .SPAN(HA.id(anchorId))
                .__space()
                ._SPAN()
                ._DIV()
                .SPAN("selectform-Label")
                .__localize("_ label.selection.thesaurus")
                ._SPAN()
                .__(addRadios(FicheQueryParser.MOTCLE_ALL_PREFIX + suffix, detailId, !withSelection, "_ label.selection.motclecondition_all"))
                .DIV(getDetailAttributes(detailId, !withSelection));
        HashSet<SubsetKey> disabledSet = new HashSet<SubsetKey>();
        if (withSelection) {
            HtmlAttributes checkAttr = HA.name(FicheQueryParser.MOTCLE_SELECTION_PREFIX + suffix).checked(true).attr("data-selectform-role", "thesaurus-checkbox");
            int thesaurusCount = subsetList.size();
            for (int i = 0; i < thesaurusCount; i++) {
                Thesaurus thesaurus = (Thesaurus) subsetList.get(i);
                SubsetKey thesaurusKey = thesaurus.getSubsetKey();
                checkAttr.value(thesaurusKey.getSubsetName());
                hp
                        .LABEL("selectform-Label")
                        .INPUT_checkbox(checkAttr)
                        .__escape(FichothequeUtils.getTitle(thesaurus, workingLang))
                        ._LABEL();
                disabledSet.add(thesaurusKey);
            }
        }
        HtmlAttributes selectAttributes = HA.id("thesaurusSelect_" + suffix).classes("selectform-ThesaurusSelect").name(FicheQueryParser.MOTCLE_SELECTION_PREFIX + suffix).multiple(true).size("10").attr("data-selectform-role", "thesaurus-select");
        hp
                .SELECT(selectAttributes)
                .__(SubsetTreeOptions.init(bdfServer.getTreeManager().getSubsetTree(SubsetKey.CATEGORY_THESAURUS), bdfServer, workingLang)
                        .multiple(true)
                        .onlyNames(true)
                        .withKeys(false)
                        .disabledSet(disabledSet)
                )
                ._SELECT()
                ._DIV()
                ._FIELDSET();
        return true;
    }

    private boolean addRadios(String name, String detailId, boolean empty, String allLocKey) {
        htmlPrinter
                .LABEL("selectform-Label")
                .INPUT_radio(HA.name(name).value("1").checked(empty))
                .__localize(allLocKey)
                ._LABEL()
                .LABEL("selectform-Label")
                .INPUT_radio(HA.name(name).value("0").checked(!empty).populate(Deploy.radio(detailId)))
                .__localize("_ label.selection.selectlist")
                ._LABEL();
        return true;
    }

    private HtmlAttributes getDetailAttributes(String detailId, boolean empty) {
        boolean hidden = ((htmlPrinter.isWithJavascript()) && (empty));
        return HA.id(detailId).classes("selectform-Detail").addClass(hidden, "hidden");
    }

    private boolean printAndOr(String name, short mode) {
        htmlPrinter
                .LABEL()
                .INPUT_radio(HA.name(name).value("and").checked((mode == ConditionsConstants.MODE_AND)))
                .__localize("_ label.global.and")
                ._LABEL()
                .__space()
                .LABEL()
                .INPUT_radio(HA.name(name).value("or").checked((mode == ConditionsConstants.MODE_OR)))
                .__localize("_ label.global.or")
                ._LABEL();
        return true;
    }

}
