/* BdfServer_HtmlProducers - Copyright (c) 2012-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.administration;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.domains.AdministrationDomain;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.commands.ResultPageBdfInstructionException;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.instruction.PermissionChecker;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public final class AdministrationHtmlProducerFactory {

    private AdministrationHtmlProducerFactory() {
    }

    public static HtmlProducer getHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        BdfServerHtmlProducer htmlProducer = getBdfHtmlProducer(parameters);
        if (htmlProducer != null) {
            htmlProducer.setBdfCommandResult(parameters.getBdfCommandResult());
        }
        return htmlProducer;
    }

    private static BdfServerHtmlProducer getBdfHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        String page = parameters.getOutput();
        BdfServer bdfServer = parameters.getBdfServer();
        RequestMap requestMap = parameters.getRequestMap();
        BdfUser bdfUser = parameters.getBdfUser();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        PermissionChecker checker = new PermissionChecker(bdfServer, bdfUser);
        switch (page) {
            case AdministrationDomain.INDEX_PAGE: {
                return new IndexHtmlProducer(checker);
            }
            case AdministrationDomain.BACKUP_PAGE: {
                checker.checkGlobalAdmin();
                return new BackupHtmlProducer(checker);
            }
            case AdministrationDomain.BACKUPRESULT_PAGE: {
                checker.checkGlobalAdmin();
                if (bdfCommandResult == null) {
                    throw new ResultPageBdfInstructionException(AdministrationDomain.BACKUPRESULT_PAGE);
                }
                String backupFileName = (String) bdfCommandResult.getResultObject(BdfInstructionConstants.BACKUPFILENAME_OBJ);
                if (backupFileName == null) {
                    throw new ResultPageBdfInstructionException(AdministrationDomain.BACKUPRESULT_PAGE);
                }
                return new BackupResultHtmlProducer(checker, backupFileName);
            }
            case AdministrationDomain.INITLOG_PAGE: {
                checker.checkGlobalAdmin();
                return new InitLogHtmlProducer(checker);
            }
            case AdministrationDomain.RESOURCES_PAGE: {
                checker.checkGlobalAdmin();
                return new ResourcesHtmlProducer(checker);
            }
            case AdministrationDomain.RESOURCE_UPLOAD_PAGE: {
                checker.checkGlobalAdmin();
                RelativePath path = getRelativePath(requestMap, bdfCommandResult);
                return new ResourceUploadHtmlProducer(checker, path);
            }
            case AdministrationDomain.ROLES_PAGE: {
                checker.checkGlobalAdmin();
                return new RolesHtmlProducer(checker);
            }
            case AdministrationDomain.SESSIONLIST_PAGE: {
                checker.checkGlobalAdmin();
                return new SessionListHtmlProducer(checker);
            }
            default:
                return null;
        }
    }

    private static RelativePath getRelativePath(RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        RelativePath path = null;
        if (bdfCommandResult != null) {
            path = (RelativePath) bdfCommandResult.getResultObject(BdfInstructionConstants.RELATIVEPATH_OBJ);
        }
        if (path == null) {
            path = BdfInstructionUtils.getMandatoryResourcePath(requestMap);
        }
        return path;
    }

}
