/* FichothequeLib_Tools - Copyright (c) 2013-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.dom;

import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;
import net.fichotheque.ExistingNameException;
import net.fichotheque.album.metadata.AlbumDim;
import net.fichotheque.album.metadata.AlbumMetadata;
import net.fichotheque.album.metadata.AlbumMetadataEditor;
import net.fichotheque.utils.AlbumUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DomMessages;
import net.mapeadores.util.xml.ElementHandler;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class AlbumMetadataDOMReader extends AbstractMetadataDOMReader {

    private final AlbumMetadataEditor albumMetadataEditor;

    public AlbumMetadataDOMReader(AlbumMetadataEditor albumMetadataEditor, MessageHandler messageHandler) {
        super(albumMetadataEditor, messageHandler, "album");
        this.albumMetadataEditor = albumMetadataEditor;
    }

    public void fillMetadata(Element element) {
        RootHandler rootHandler = new RootHandler("/" + element.getTagName());
        DOMUtils.readChildren(element, rootHandler);
        rootHandler.flush();
    }


    private class RootHandler implements ElementHandler {

        private final String parentXpath;
        private final Map<String, String> usageMap = new HashMap<String, String>();

        private RootHandler(String parentXpath) {
            this.parentXpath = parentXpath;
        }

        @Override
        public void readElement(Element element) {
            if (readCommonElement(element, parentXpath)) {
                return;
            }
            String tagname = element.getTagName();
            String xpath = parentXpath + "/" + tagname;
            if (tagname.equals("album-dim")) {
                addAlbumDim(element, xpath);
            } else if (tagname.equals("dim-usage")) {
                appendDimUsage(element, xpath);
            } else {
                DomMessages.unknownTagWarning(messageHandler, xpath);
            }

        }

        private void addAlbumDim(Element element, String xpath) {
            String name = element.getAttribute("name");
            if (name.length() == 0) {
                DomMessages.emptyAttribute(messageHandler, xpath, "name");
                return;
            }
            xpath = xpath + " name=\"" + name + "\"";
            String type = element.getAttribute("type");
            if (type.isEmpty()) {
                DomMessages.emptyAttribute(messageHandler, xpath, "type");
                return;
            }
            try {
                type = AlbumUtils.checkDimType(type);
            } catch (IllegalArgumentException iae) {
                DomMessages.wrongAttributeValue(messageHandler, xpath, "type", type);
                return;
            }
            int width = 0;
            int height = 0;
            if (AlbumUtils.needWidth(type)) {
                width = getInt(element, xpath, "width");
                if (width == -1) {
                    return;
                }
            }
            if (AlbumUtils.needHeight(type)) {
                height = getInt(element, xpath, "height");
                if (height == -1) {
                    return;
                }
            }
            AlbumDim albumDim;
            try {
                albumDim = albumMetadataEditor.createAlbumDim(name, type);
            } catch (ExistingNameException | ParseException e) {
                return;
            }
            albumMetadataEditor.setDim(albumDim, width, height);

        }

        private int getInt(Element element, String xpath, String attributeName) {
            String intString = element.getAttribute(attributeName);
            int result;
            if (intString.length() == 0) {
                DomMessages.emptyAttribute(messageHandler, xpath, attributeName);
                return -1;
            }
            try {
                result = Integer.parseInt(intString);
            } catch (NumberFormatException nfe) {
                DomMessages.wrongIntegerAttributeValue(messageHandler, xpath, attributeName, intString);
                return -1;
            }
            if (result < 1) {
                DomMessages.wrongIntegerAttributeValue(messageHandler, xpath, attributeName, intString);
                return -1;
            }
            return result;
        }

        private void appendDimUsage(Element element, String xpath) {
            String usageName = element.getAttribute("usage-name");
            if (usageName.length() == 0) {
                DomMessages.emptyAttribute(messageHandler, xpath, "usage-name");
                return;
            }
            String dimName = element.getAttribute("dim-name");
            if (dimName.length() == 0) {
                DomMessages.emptyAttribute(messageHandler, xpath, "dim-name");
                return;
            }
            usageMap.put(usageName, dimName);
        }

        private void flush() {
            AlbumMetadata metadata = (AlbumMetadata) albumMetadataEditor.getMetadata();
            for (Map.Entry<String, String> mapEntry : usageMap.entrySet()) {
                String usageName = mapEntry.getKey();
                String dimName = mapEntry.getValue();
                AlbumDim albumDim = metadata.getAlbumDimByName(dimName);
                if (albumDim != null) {
                    albumMetadataEditor.setAlbumDimForUsage(usageName, albumDim);
                }
            }
        }

    }


}
