/* FichothequeLib_Tools - Copyright (c) 2007-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.dom;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.CorpusMetadata;
import net.fichotheque.corpus.metadata.CorpusMetadataEditor;
import net.fichotheque.corpus.metadata.ExistingFieldKeyException;
import net.fichotheque.corpus.metadata.FieldGeneration;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.corpus.metadata.FieldOptionException;
import net.fichotheque.corpus.metadata.FieldOptionsConstants;
import net.fichotheque.tools.corpus.FieldGenerationParser;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.fichotheque.utils.FieldOptionsUtils;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.logging.SimpleLineMessageHandler;
import net.mapeadores.util.money.ExtendedCurrency;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DomMessages;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;


/**
 * Lit le format XML de corpus-metadata à partir du DOM.
 *
 * @author Vincent Calame
 */
public class CorpusMetadataDOMReader extends AbstractMetadataDOMReader {

    private final CorpusMetadataEditor corpusMetadataEditor;

    public CorpusMetadataDOMReader(CorpusMetadataEditor corpusMetadataEditor, MessageHandler messageHandler) {
        super(corpusMetadataEditor, messageHandler, "corpus");
        this.corpusMetadataEditor = corpusMetadataEditor;
    }

    public void fillMetadata(Element element) {
        RootHandler rootHandler = new RootHandler("/" + element.getTagName());
        DOMUtils.readChildren(element, rootHandler);
        rootHandler.flush();
    }

    private FieldKey getFieldKey(short category, Element element) {
        String fieldName = element.getAttribute("name");
        if (fieldName.length() == 0) {
            fieldName = element.getAttribute("type");
        }
        if (fieldName.length() == 0) {
            DomMessages.emptyAttribute(messageHandler, element.getTagName(), "name");
            return null;
        }
        try {
            return FieldKey.parse(category, fieldName);
        } catch (ParseException e) {
            DomMessages.wrongAttributeValue(messageHandler, element.getTagName(), "name", fieldName);
            return null;
        }
    }

    private short getFicheItemType(FieldKey fieldKey, Element element) {
        String ficheItem = element.getAttribute("fiche-item");
        if (ficheItem.length() > 0) {
            try {
                return CorpusMetadataUtils.ficheItemTypeToShort(ficheItem);
            } catch (IllegalArgumentException iae) {
                addFichothequeError("_ error.wrong.ficheitemtype", ficheItem + " (" + fieldKey.toString() + ")");
                return CorpusField.ITEM_FIELD;
            }
        }
        return CorpusField.ITEM_FIELD;
    }


    private CorpusField getCorpusField(Element element) {
        String fieldkey_string = element.getAttribute("field-key");
        if (fieldkey_string.length() == 0) {
            DomMessages.emptyAttribute(messageHandler, element.getTagName(), "field-key");
            return null;
        }
        FieldKey fieldKey;
        try {
            fieldKey = FieldKey.parse(fieldkey_string);
        } catch (ParseException pe) {
            DomMessages.wrongAttributeValue(messageHandler, element.getTagName(), "field-key", fieldkey_string);
            return null;
        }
        CorpusField corpusField = ((CorpusMetadata) corpusMetadataEditor.getMetadata()).getCorpusField(fieldKey);
        if (corpusField == null) {
            addFichothequeError("_ error.unknown.fieldkey", fieldKey);
        }
        return corpusField;
    }

    private void addFichothequeError(String messageKey, Object messageValue) {
        messageHandler.addMessage(FichothequeConstants.SEVERE_FICHOTHEQUE, LocalisationUtils.toMessage(messageKey, messageValue));
    }


    private class RootHandler implements ElementHandler {

        private final String parentXpath;
        private final StringBuilder fieldGenerationRawBuf = new StringBuilder();
        private Element fieldsElement = null;
        private Element optionsElement = null;
        private Element l10nElement = null;

        private RootHandler(String parentXpath) {
            this.parentXpath = parentXpath;
        }

        @Override
        public void readElement(Element element) {
            if (readCommonElement(element, parentXpath)) {
                return;
            }
            String tagname = element.getTagName();
            if ((fieldsElement == null) && (tagname.equals("fields"))) {
                fieldsElement = element;
            } else if (tagname.equals("field-generation")) {
                fieldGenerationRawBuf.append(XMLUtils.getRawData(element));
            } else if ((optionsElement == null) && (tagname.equals("options"))) {
                optionsElement = element;
            } else if ((l10nElement == null) && (tagname.equals("l10n"))) {
                l10nElement = element;
            } else {
                DomMessages.unknownTagWarning(messageHandler, tagname);
            }
        }

        private void flush() {
            FieldsHandler fieldsHandler = new FieldsHandler();
            DOMUtils.readChildren(fieldsElement, fieldsHandler);
            if (optionsElement != null) {
                OptionsHandler optionsHandler = new OptionsHandler(fieldGenerationRawBuf);
                DOMUtils.readChildren(optionsElement, optionsHandler);
            }
            if (l10nElement != null) {
                L10nHandler l10nHandler = new L10nHandler();
                DOMUtils.readChildren(l10nElement, l10nHandler);
            }
            if (fieldGenerationRawBuf.length() > 0) {
                FieldGeneration fieldGeneration = FieldGenerationParser.parse(fieldGenerationRawBuf.toString(), new SimpleLineMessageHandler());
                corpusMetadataEditor.setFieldGeneration(fieldGeneration);
            }
        }

    }


    private class FieldsHandler implements ElementHandler {

        private FieldsHandler() {

        }

        @Override
        public void readElement(Element element) {
            String tagname = element.getTagName();
            FieldKey fieldKey = null;
            short ficheItemType = CorpusField.NO_FICHEITEM_FIELD;
            if (tagname.equals("soustitre")) {
                fieldKey = FieldKey.SOUSTITRE;
            } else if (tagname.equals("texte")) {
                fieldKey = FieldKey.build(FieldKey.SECTION_CATEGORY, "texte");
            } else if ((tagname.equals("section")) || (tagname.equals("annexe"))) {
                fieldKey = getFieldKey(FieldKey.SECTION_CATEGORY, element);
            } else if (tagname.equals("propriete")) {
                fieldKey = getFieldKey(FieldKey.PROPRIETE_CATEGORY, element);
                if (fieldKey != null) {
                    ficheItemType = getFicheItemType(fieldKey, element);
                }
            } else if (tagname.equals("information")) {
                fieldKey = getFieldKey(FieldKey.INFORMATION_CATEGORY, element);
                if (fieldKey != null) {
                    ficheItemType = getFicheItemType(fieldKey, element);
                }
            } else {
                DomMessages.unknownTagWarning(messageHandler, tagname);
            }
            if (fieldKey != null) {
                try {
                    corpusMetadataEditor.createCorpusField(fieldKey, ficheItemType);
                } catch (ExistingFieldKeyException efke) {
                    addFichothequeError("_ error.existing.fieldkey", fieldKey.toString());
                }
            }
        }

    }


    private class L10nHandler implements ElementHandler {

        private L10nHandler() {

        }

        @Override
        public void readElement(Element element) {
            String tagname = element.getTagName();
            if (tagname.equals("field")) {
                CorpusField corpusField = getCorpusField(element);
                if (corpusField == null) {
                    return;
                }
                FieldL10nHandler fieldL10nHandler = new FieldL10nHandler(corpusField);
                DOMUtils.readChildren(element, fieldL10nHandler);
            } else if (tagname.equals("intitule")) {
                FichothequeDOMUtils.readIntitule(element, corpusMetadataEditor, messageHandler, "corpus");
            }
        }

    }


    private class FieldL10nHandler implements ElementHandler {

        private final CorpusField corpusField;

        private FieldL10nHandler(CorpusField corpusField) {
            this.corpusField = corpusField;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if ((tagName.equals("label")) || (tagName.equals("lib"))) {
                try {
                    Label label = LabelUtils.readLabel(element);
                    if (label != null) {
                        corpusMetadataEditor.putFieldLabel(corpusField, label);
                    }
                } catch (ParseException ile) {
                }
            }
        }

    }


    private class OptionsHandler implements ElementHandler {

        private final StringBuilder fieldGenerationSourceBuf;

        private OptionsHandler(StringBuilder fieldGenerationSourceBuf) {
            this.fieldGenerationSourceBuf = fieldGenerationSourceBuf;
        }

        @Override
        public void readElement(Element element) {
            String tagname = element.getTagName();
            if (tagname.equals("option")) {
                String name = element.getAttribute("name");
                if (name.length() == 0) {
                    DomMessages.emptyAttribute(messageHandler, "option", "name");
                    return;
                }
                CorpusField corpusField = getCorpusField(element);
                if (corpusField == null) {
                    return;
                }
                if (!FieldOptionsUtils.testFieldOption(name, corpusField)) {
                    addFichothequeError("_ error.unsupported.fieldoption", name);
                    return;
                }
                if (name.equals(FieldOptionsConstants.INFORMATIONDISPLAY_OPTION)) {
                    initInformationDisplay(corpusField, element);
                } else if (name.equals(FieldOptionsConstants.DEFAULTSPHEREKEY_OPTION)) {
                    initDefaultSphereKey(corpusField, element);
                } else if ((name.equals("mainPersonne") || (name.equals(FieldOptionsConstants.SUBFIELDDISPLAY_OPTION)))) {
                    initSubfieldDisplay(corpusField, element);
                } else if (name.equals("titreGenerationFormat")) {
                    initOldFieldGeneration(corpusField, element, fieldGenerationSourceBuf);
                } else if (name.equals(FieldOptionsConstants.CURRENCYARRAY_OPTION)) {
                    initCurrencies(corpusField, element);
                } else if (name.equals(FieldOptionsConstants.GEOLOCALISATIONFIELD_OPTION)) {
                    initGeolocalisationField(corpusField);
                } else if (name.equals(FieldOptionsConstants.ADDRESSFIELDARRAY_OPTION)) {
                    initAddressFieldArray(corpusField, element);
                } else if (name.equals(FieldOptionsConstants.LANGSCOPE_OPTION)) {
                    initLangScope(corpusField, element);
                } else if (name.equals(FieldOptionsConstants.LANGARRAY_OPTION)) {
                    initLangs(corpusField, element);
                } else {
                    initStringOption(corpusField, element, name);
                }
            } else {
                DomMessages.unknownTagWarning(messageHandler, tagname);
            }
        }

        private void initInformationDisplay(CorpusField corpusField, Element el) {
            String value = el.getAttribute("value");
            Short informationDisplay = null;
            if (value.length() == 0) {
                informationDisplay = FieldOptionsUtils.getDefaultInformationDisplay();
            } else {
                try {
                    informationDisplay = FieldOptionsUtils.informationDisplayToShort(value);
                } catch (IllegalArgumentException iae) {
                    DomMessages.wrongAttributeValue(messageHandler, el.getTagName(), "value", value);
                }
            }
            if (informationDisplay != null) {
                try {
                    corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.INFORMATIONDISPLAY_OPTION, informationDisplay);
                } catch (FieldOptionException foe) {
                    throw new ImplementationException(foe);
                }
            }
        }

        private void initGeolocalisationField(CorpusField corpusField) {
            corpusMetadataEditor.setGeolocalisationField(corpusField);
        }

        private void initDefaultSphereKey(CorpusField corpusField, Element el) {
            String value = el.getAttribute("value");
            SubsetKey sphereKey = null;
            if (!value.equals("null")) {
                try {
                    sphereKey = SubsetKey.parse(value);

                } catch (ParseException pe) {
                    DomMessages.wrongAttributeValue(messageHandler, el.getTagName(), "value", value);
                    return;
                }
                if (!sphereKey.isSphereSubset()) {
                    DomMessages.wrongAttributeValue(messageHandler, el.getTagName(), "value", value);
                    return;
                }

            }
            try {
                corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.DEFAULTSPHEREKEY_OPTION, sphereKey);
            } catch (FieldOptionException foe) {
                throw new ImplementationException(foe);
            }
        }

        private void initSubfieldDisplay(CorpusField corpusField, Element el) {
            if (StringUtils.isTrue(el.getAttribute("value"))) {
                try {
                    corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.SUBFIELDDISPLAY_OPTION, Boolean.TRUE);
                } catch (FieldOptionException foe) {
                    throw new ImplementationException(foe);
                }
            }
        }

        private void initOldFieldGeneration(CorpusField corpusField, Element el, StringBuilder fieldGenerationSourceBuf) {
            List<String> valueList = readValueList(el);
            if (valueList == null) {
                return;
            }
            if (fieldGenerationSourceBuf.length() > 0) {
                fieldGenerationSourceBuf.append("\n\n");
            }
            fieldGenerationSourceBuf.append(corpusField.getFieldKey().toString());
            fieldGenerationSourceBuf.append('\n');
            int size = valueList.size();
            fieldGenerationSourceBuf.append(valueList.get(0));
            fieldGenerationSourceBuf.append('\n');
            if (size > 1) {
                String value2 = valueList.get(1).trim();
                if (value2.length() == 0) {
                    fieldGenerationSourceBuf.append("-");
                } else {
                    fieldGenerationSourceBuf.append(value2);
                }
                fieldGenerationSourceBuf.append('\n');
                if (size > 2) {
                    fieldGenerationSourceBuf.append(valueList.get(2));
                }
            }
        }

        private void initAddressFieldArray(CorpusField corpusField, Element el) {
            List<String> valueList = readValueList(el);
            if (valueList == null) {
                return;
            }
            List<FieldKey> fieldKeyList = new ArrayList<FieldKey>();
            for (String value : valueList) {
                boolean done;
                try {
                    FieldKey otherFieldKey = FieldKey.parse(value);
                    CorpusField otherCorpusField = ((CorpusMetadata) corpusMetadataEditor.getMetadata()).getCorpusField(otherFieldKey);
                    if (otherCorpusField != null) {
                        fieldKeyList.add(otherFieldKey);
                        done = true;
                    } else {
                        done = false;
                    }
                } catch (ParseException pe) {
                    done = false;
                }
                if (!done) {
                    DomMessages.wrongElementValue(messageHandler, "value", value);
                }
            }
            try {
                corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.ADDRESSFIELDARRAY_OPTION, fieldKeyList.toArray(new FieldKey[fieldKeyList.size()]));
            } catch (FieldOptionException foe) {
                throw new ImplementationException(foe);
            }
        }

        private void initStringOption(CorpusField corpusField, Element el, String name) {
            if (name.equals("imageSource") || name.equals("albumKey")) {
                return;
            }
            String value = el.getAttribute("value");
            if (value.length() > 0) {
                try {
                    corpusMetadataEditor.setFieldOption(corpusField, name, value);
                } catch (FieldOptionException foe) {
                    throw new ImplementationException(foe);
                }
            }
        }

        private void initCurrencies(CorpusField corpusField, Element el) {
            List<String> valueList = readValueList(el);
            if (valueList == null) {
                return;
            }
            List<ExtendedCurrency> currencyList = new ArrayList<ExtendedCurrency>();
            for (String value : valueList) {
                try {
                    currencyList.add(ExtendedCurrency.parse(value));
                } catch (ParseException pe) {
                    DomMessages.wrongElementValue(messageHandler, "value", value);
                }
            }
            if (currencyList.isEmpty()) {
                return;
            }
            try {
                corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.CURRENCYARRAY_OPTION, currencyList.toArray(new ExtendedCurrency[currencyList.size()]));
            } catch (FieldOptionException foe) {
                throw new ImplementationException(foe);
            }
        }

        private void initLangScope(CorpusField corpusField, Element el) {
            String value = el.getAttribute("value");
            Short langScope = null;
            if (value.length() == 0) {
                langScope = FieldOptionsUtils.getDefaultLangScope(corpusField.getFieldKey());
            } else {
                try {
                    langScope = FieldOptionsUtils.langScopeToShort(value);
                } catch (IllegalArgumentException iae) {
                    DomMessages.wrongAttributeValue(messageHandler, el.getTagName(), "value", value);
                }
            }
            if (langScope != null) {
                try {
                    corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.LANGSCOPE_OPTION, langScope);
                } catch (FieldOptionException foe) {
                    throw new ImplementationException(foe);
                }
            }
        }

        private void initLangs(CorpusField corpusField, Element el) {
            List<String> valueList = readValueList(el);
            if (valueList == null) {
                return;
            }
            List<Lang> langList = new ArrayList<Lang>();
            for (String value : valueList) {
                try {
                    langList.add(Lang.parse(value));
                } catch (ParseException pe) {
                    DomMessages.wrongElementValue(messageHandler, "value", value);
                }
            }
            if (langList.isEmpty()) {
                return;
            }
            try {
                corpusMetadataEditor.setFieldOption(corpusField, FieldOptionsConstants.LANGARRAY_OPTION, langList.toArray(new Lang[langList.size()]));
            } catch (FieldOptionException foe) {
                throw new ImplementationException(foe);
            }
        }

        private List<String> readValueList(Element element) {
            List<String> valueList = new ArrayList<String>();
            ValueHandler valueHandler = new ValueHandler(valueList);
            DOMUtils.readChildren(element, valueHandler);
            int size = valueList.size();
            if (size == 0) {
                DomMessages.missingChildTag(messageHandler, element.getTagName(), "value");
                return null;
            }
            return valueList;
        }

    }


    private class ValueHandler implements ElementHandler {

        private final List<String> valueList;

        private ValueHandler(List<String> valueList) {
            this.valueList = valueList;
        }

        @Override
        public void readElement(Element el) {
            String tagname = el.getTagName();
            if (tagname.equals("value")) {
                valueList.add(XMLUtils.getData(el));
            } else {
                DomMessages.unknownTagWarning(messageHandler, tagname);
            }
        }

    }

}
