/* FichothequeLib_Tools - Copyright (c) 2012-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import net.fichotheque.SubsetKey;
import net.fichotheque.album.Album;
import net.fichotheque.album.Illustration;
import net.fichotheque.croisement.Croisement;
import net.fichotheque.croisement.LiaisonSortKey;
import net.fichotheque.croisement.Lien;
import net.fichotheque.extraction.def.ExtraitAlbumDef;
import net.fichotheque.extraction.def.IllustrationFilter;
import net.fichotheque.extraction.run.AlbumExtractor;
import net.fichotheque.extraction.run.ExtraitAlbumResult;
import net.fichotheque.extraction.run.IllustrationExtractInfo;
import net.fichotheque.utils.CroisementUtils;


/**
 *
 * @author Vincent Calame
 */
class AlbumExtractorImpl implements AlbumExtractor {

    private final Map<SubsetKey, ResultBuilder> builderMap = new TreeMap<SubsetKey, ResultBuilder>();
    private final ExtraitAlbumDef extraitAlbumDef;

    AlbumExtractorImpl(ExtraitAlbumDef extraitAlbumDef) {
        this.extraitAlbumDef = extraitAlbumDef;
    }

    @Override
    public void add(Illustration illustration, Croisement croisement) {
        Album album = illustration.getAlbum();
        ResultBuilder resultBuilder = builderMap.get(album.getSubsetKey());
        IllustrationFilter illustrationFilter = extraitAlbumDef.getIllustrationFilter();
        if (resultBuilder == null) {
            resultBuilder = new ResultBuilder(album);
            builderMap.put(album.getSubsetKey(), resultBuilder);
        }
        resultBuilder.add(illustration, illustrationFilter, croisement);
    }

    @Override
    public boolean containsIllustration() {
        return (!builderMap.isEmpty());
    }

    @Override
    public ExtraitAlbumResult getExtraitAlbumResult() {
        int size = builderMap.size();
        Album[] albumArray = new Album[size];
        List<IllustrationExtractInfo>[] illustrationExtractInfoListArray = new List[size];
        int p = 0;
        for (ResultBuilder resultBuilder : builderMap.values()) {
            albumArray[p] = resultBuilder.album;
            illustrationExtractInfoListArray[p] = resultBuilder.getIllustrationExtractInfoList();
            p++;
        }
        InternalExtraitAlbumResult extraitAlbumResult = new InternalExtraitAlbumResult(extraitAlbumDef, albumArray, illustrationExtractInfoListArray);
        return extraitAlbumResult;
    }


    private class ResultBuilder {

        private Album album;
        private final SortedMap<LiaisonSortKey, InternalIllustrationExtractInfo> map = new TreeMap<LiaisonSortKey, InternalIllustrationExtractInfo>();

        private ResultBuilder(Album album) {
            this.album = album;
        }

        private void add(Illustration illustration, IllustrationFilter illustrationFilter, Croisement croisement) {
            int position;
            if (croisement != null) {
                Lien lien = croisement.getLienList().get(0);
                position = CroisementUtils.getPosition(illustration, croisement.getCroisementKey(), lien);
            } else {
                position = map.size() + 1;
            }
            LiaisonSortKey key = new LiaisonSortKey(illustration.getId(), position);
            map.put(key, new InternalIllustrationExtractInfo(illustration, illustrationFilter, croisement));
        }

        private List<IllustrationExtractInfo> getIllustrationExtractInfoList() {
            return new ArrayList(map.values());
        }

    }


    private static class InternalExtraitAlbumResult implements ExtraitAlbumResult {

        private final Album[] albumArray;
        private final List<IllustrationExtractInfo>[] illustrationExtractInfoListArray;
        private final ExtraitAlbumDef extraitAlbumDef;

        private InternalExtraitAlbumResult(ExtraitAlbumDef extraitAlbumDef, Album[] albumArray, List<IllustrationExtractInfo>[] illustrationExtractInfoListArray) {
            this.extraitAlbumDef = extraitAlbumDef;
            this.albumArray = albumArray;
            this.illustrationExtractInfoListArray = illustrationExtractInfoListArray;
        }

        @Override
        public ExtraitAlbumDef getExtraitAlbumDef() {
            return extraitAlbumDef;
        }

        @Override
        public int getAlbumCount() {
            return albumArray.length;
        }

        @Override
        public Album getAlbum(int index) {
            return albumArray[index];
        }

        @Override
        public List<IllustrationExtractInfo> getIllustrationExtractInfoList(int index) {
            return illustrationExtractInfoListArray[index];
        }

    }


    private static class InternalIllustrationExtractInfo implements IllustrationExtractInfo {

        private final Illustration illustration;
        private final IllustrationFilter illustrationFilter;
        private final Croisement croisement;

        public InternalIllustrationExtractInfo(Illustration illustration, IllustrationFilter illustrationFilter, Croisement croisement) {
            this.illustration = illustration;
            this.illustrationFilter = illustrationFilter;
            this.croisement = croisement;
        }

        @Override
        public Illustration getIllustration() {
            return illustration;
        }

        @Override
        public IllustrationFilter getIllustrationFilter() {
            return illustrationFilter;
        }

        @Override
        public Croisement getCroisement() {
            return croisement;
        }

    }

}
